@@ -37,7 +37,9 @@
  *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA.
 \*****************************************************************************/
 
+#include "src/common/hash.h"
 #include "src/common/slurm_jobacct_gather.h"
+#include "src/common/slurm_protocol_pack.h"
 #include "src/common/slurmdb_pack.h"
 #include "src/common/slurmdbd_defs.h"
 #include "src/common/slurmdbd_pack.h"
@@ -429,10 +431,15 @@ static void _pack_job_start_msg(void *in, uint16_t rpc_version, buf_t *buffer)
 {
 	dbd_job_start_msg_t *msg = (dbd_job_start_msg_t *)in;
 
-	if (msg->script_buf)
-		msg->script = msg->script_buf->head;
+	/*
+	 * Generate node_inx outside of locks -- not _setup_job_start_msg().
+	 * See slurmdbd/acct_storage_p_node_inx() for need to
+	 * use_create_node_inx().
+	 */
+	xassert(!msg->node_inx);
+	msg->node_inx = acct_storage_g_node_inx(NULL, msg->nodes);
 
-	if (rpc_version >= SLURM_21_08_PROTOCOL_VERSION) {
+	if (rpc_version >= SLURM_22_05_PROTOCOL_VERSION) {
 		packstr(msg->account, buffer);
 		pack32(msg->alloc_nodes, buffer);
 		pack32(msg->array_job_id, buffer);
@@ -446,7 +453,6 @@ static void _pack_job_start_msg(void *in, uint16_t rpc_version, buf_t *buffer)
 		pack32(msg->db_flags, buffer);
 		pack64(msg->db_index, buffer);
 		pack_time(msg->eligible_time, buffer);
-		packstr(msg->env, buffer);
 		pack32(msg->gid, buffer);
 		packstr(msg->gres_used, buffer);
 		pack32(msg->job_id, buffer);
@@ -464,7 +470,6 @@ static void _pack_job_start_msg(void *in, uint16_t rpc_version, buf_t *buffer)
 		pack32(msg->req_cpus, buffer);
 		pack64(msg->req_mem, buffer);
 		pack32(msg->resv_id, buffer);
-		packstr(msg->script, buffer);
 		pack_time(msg->start_time, buffer);
 		packstr(msg->submit_line, buffer);
 		pack_time(msg->submit_time, buffer);
@@ -474,7 +479,9 @@ static void _pack_job_start_msg(void *in, uint16_t rpc_version, buf_t *buffer)
 		pack32(msg->uid, buffer);
 		packstr(msg->wckey, buffer);
 		packstr(msg->work_dir, buffer);
-	} else if (rpc_version >= SLURM_20_11_PROTOCOL_VERSION) {
+		packstr(msg->env_hash, buffer);
+		packstr(msg->script_hash, buffer);
+	} else if (rpc_version >= SLURM_21_08_PROTOCOL_VERSION) {
 		packstr(msg->account, buffer);
 		pack32(msg->alloc_nodes, buffer);
 		pack32(msg->array_job_id, buffer);
@@ -484,9 +491,11 @@ static void _pack_job_start_msg(void *in, uint16_t rpc_version, buf_t *buffer)
 		pack32(msg->array_task_pending, buffer);
 		pack32(msg->assoc_id, buffer);
 		packstr(msg->constraints, buffer);
+		packstr(msg->container, buffer);
 		pack32(msg->db_flags, buffer);
 		pack64(msg->db_index, buffer);
 		pack_time(msg->eligible_time, buffer);
+		packnull(buffer);
 		pack32(msg->gid, buffer);
 		packstr(msg->gres_used, buffer);
 		pack32(msg->job_id, buffer);
@@ -504,7 +513,9 @@ static void _pack_job_start_msg(void *in, uint16_t rpc_version, buf_t *buffer)
 		pack32(msg->req_cpus, buffer);
 		pack64(msg->req_mem, buffer);
 		pack32(msg->resv_id, buffer);
+		packnull(buffer);
 		pack_time(msg->start_time, buffer);
+		packstr(msg->submit_line, buffer);
 		pack_time(msg->submit_time, buffer);
 		pack32(msg->timelimit, buffer);
 		packstr(msg->tres_alloc_str, buffer);
@@ -526,8 +537,6 @@ static void _pack_job_start_msg(void *in, uint16_t rpc_version, buf_t *buffer)
 		pack64(msg->db_index, buffer);
 		pack_time(msg->eligible_time, buffer);
 		pack32(msg->gid, buffer);
-		packnull(buffer);
-		packnull(buffer);
 		packstr(msg->gres_used, buffer);
 		pack32(msg->job_id, buffer);
 		pack32(msg->job_state, buffer);
@@ -553,9 +562,6 @@ static void _pack_job_start_msg(void *in, uint16_t rpc_version, buf_t *buffer)
 		packstr(msg->wckey, buffer);
 		packstr(msg->work_dir, buffer);
 	}
-
-	if (msg->script_buf)
-		msg->script = NULL;
 }
 
 static int _unpack_job_start_msg(void **msg, uint16_t rpc_version,
@@ -568,7 +574,7 @@ static int _unpack_job_start_msg(void **msg, uint16_t rpc_version,
 	msg_ptr->array_job_id = 0;
 	msg_ptr->array_task_id = NO_VAL;
 
-	if (rpc_version >= SLURM_21_08_PROTOCOL_VERSION) {
+	if (rpc_version >= SLURM_22_05_PROTOCOL_VERSION) {
 		safe_unpackstr_xmalloc(&msg_ptr->account, &uint32_tmp, buffer);
 		safe_unpack32(&msg_ptr->alloc_nodes, buffer);
 		safe_unpack32(&msg_ptr->array_job_id, buffer);
@@ -585,7 +591,6 @@ static int _unpack_job_start_msg(void **msg, uint16_t rpc_version,
 		safe_unpack32(&msg_ptr->db_flags, buffer);
 		safe_unpack64(&msg_ptr->db_index, buffer);
 		safe_unpack_time(&msg_ptr->eligible_time, buffer);
-		safe_unpackstr_xmalloc(&msg_ptr->env, &uint32_tmp, buffer);
 		safe_unpack32(&msg_ptr->gid, buffer);
 		safe_unpackstr_xmalloc(&msg_ptr->gres_used, &uint32_tmp,
 				       buffer);
@@ -606,7 +611,6 @@ static int _unpack_job_start_msg(void **msg, uint16_t rpc_version,
 		safe_unpack32(&msg_ptr->req_cpus, buffer);
 		safe_unpack64(&msg_ptr->req_mem, buffer);
 		safe_unpack32(&msg_ptr->resv_id, buffer);
-		safe_unpackstr_xmalloc(&msg_ptr->script, &uint32_tmp, buffer);
 		safe_unpack_time(&msg_ptr->start_time, buffer);
 		safe_unpackstr_xmalloc(&msg_ptr->submit_line,
 				       &uint32_tmp, buffer);
@@ -619,7 +623,14 @@ static int _unpack_job_start_msg(void **msg, uint16_t rpc_version,
 		safe_unpack32(&msg_ptr->uid, buffer);
 		safe_unpackstr_xmalloc(&msg_ptr->wckey, &uint32_tmp, buffer);
 		safe_unpackstr_xmalloc(&msg_ptr->work_dir, &uint32_tmp, buffer);
-	} else if (rpc_version >= SLURM_20_11_PROTOCOL_VERSION) {
+		safe_unpackstr_xmalloc(&msg_ptr->env_hash, &uint32_tmp, buffer);
+		safe_unpackstr_xmalloc(&msg_ptr->script_hash,
+				       &uint32_tmp, buffer);
+	} else if (rpc_version >= SLURM_21_08_PROTOCOL_VERSION) {
+		char *env, *script;
+		slurm_hash_t hash = {
+			.type = HASH_PLUGIN_K12,
+		};
 		safe_unpackstr_xmalloc(&msg_ptr->account, &uint32_tmp, buffer);
 		safe_unpack32(&msg_ptr->alloc_nodes, buffer);
 		safe_unpack32(&msg_ptr->array_job_id, buffer);
@@ -631,9 +642,12 @@ static int _unpack_job_start_msg(void **msg, uint16_t rpc_version,
 		safe_unpack32(&msg_ptr->assoc_id, buffer);
 		safe_unpackstr_xmalloc(&msg_ptr->constraints,
 				       &uint32_tmp, buffer);
+		safe_unpackstr_xmalloc(&msg_ptr->container,
+				       &uint32_tmp, buffer);
 		safe_unpack32(&msg_ptr->db_flags, buffer);
 		safe_unpack64(&msg_ptr->db_index, buffer);
 		safe_unpack_time(&msg_ptr->eligible_time, buffer);
+		safe_unpackstr_xmalloc(&env, &uint32_tmp, buffer);
 		safe_unpack32(&msg_ptr->gid, buffer);
 		safe_unpackstr_xmalloc(&msg_ptr->gres_used, &uint32_tmp,
 				       buffer);
@@ -654,7 +668,10 @@ static int _unpack_job_start_msg(void **msg, uint16_t rpc_version,
 		safe_unpack32(&msg_ptr->req_cpus, buffer);
 		safe_unpack64(&msg_ptr->req_mem, buffer);
 		safe_unpack32(&msg_ptr->resv_id, buffer);
+		safe_unpackstr_xmalloc(&script, &uint32_tmp, buffer);
 		safe_unpack_time(&msg_ptr->start_time, buffer);
+		safe_unpackstr_xmalloc(&msg_ptr->submit_line,
+				       &uint32_tmp, buffer);
 		safe_unpack_time(&msg_ptr->submit_time, buffer);
 		safe_unpack32(&msg_ptr->timelimit, buffer);
 		safe_unpackstr_xmalloc(&msg_ptr->tres_alloc_str,
@@ -664,8 +681,21 @@ static int _unpack_job_start_msg(void **msg, uint16_t rpc_version,
 		safe_unpack32(&msg_ptr->uid, buffer);
 		safe_unpackstr_xmalloc(&msg_ptr->wckey, &uint32_tmp, buffer);
 		safe_unpackstr_xmalloc(&msg_ptr->work_dir, &uint32_tmp, buffer);
+		if (env) {
+			(void) hash_g_compute(env, strlen(env),
+					      NULL, 0, &hash);
+			msg_ptr->env_hash = xstrdup_printf(
+				"%c:%s", hash.type, hash.hash);
+			xfree(env);
+		}
+		if (script) {
+			(void) hash_g_compute(script, strlen(script),
+					      NULL, 0, &hash);
+			msg_ptr->script_hash = xstrdup_printf(
+				"%c:%s", hash.type, hash.hash);
+			xfree(script);
+		}
 	} else if (rpc_version >= SLURM_MIN_PROTOCOL_VERSION) {
-		char *tmp_ptr = NULL;
 		safe_unpackstr_xmalloc(&msg_ptr->account, &uint32_tmp, buffer);
 		safe_unpack32(&msg_ptr->alloc_nodes, buffer);
 		safe_unpack32(&msg_ptr->array_job_id, buffer);
@@ -681,10 +711,6 @@ static int _unpack_job_start_msg(void **msg, uint16_t rpc_version,
 		safe_unpack64(&msg_ptr->db_index, buffer);
 		safe_unpack_time(&msg_ptr->eligible_time, buffer);
 		safe_unpack32(&msg_ptr->gid, buffer);
-		safe_unpackstr_xmalloc(&tmp_ptr, &uint32_tmp, buffer);
-		xfree(tmp_ptr);
-		safe_unpackstr_xmalloc(&tmp_ptr, &uint32_tmp, buffer);
-		xfree(tmp_ptr);
 		safe_unpackstr_xmalloc(&msg_ptr->gres_used, &uint32_tmp,
 				       buffer);
 		safe_unpack32(&msg_ptr->job_id, buffer);
@@ -725,6 +751,49 @@ unpack_error:
 	return SLURM_ERROR;
 }
 
+static void _pack_job_heavy_msg(void *in, uint16_t rpc_version, buf_t *buffer)
+{
+	dbd_job_heavy_msg_t *msg = (dbd_job_heavy_msg_t *)in;
+
+	if (msg->script_buf)
+		msg->script = msg->script_buf->head;
+
+	if (rpc_version >= SLURM_22_05_PROTOCOL_VERSION) {
+		packstr(msg->env, buffer);
+		packstr(msg->env_hash, buffer);
+		packstr(msg->script, buffer);
+		packstr(msg->script_hash, buffer);
+	}
+
+	if (msg->script_buf)
+		msg->script = NULL;
+}
+
+static int _unpack_job_heavy_msg(void **msg, uint16_t rpc_version,
+				  buf_t *buffer)
+{
+	uint32_t uint32_tmp;
+	dbd_job_heavy_msg_t *msg_ptr = xmalloc(sizeof(*msg_ptr));
+	*msg = msg_ptr;
+
+	if (rpc_version >= SLURM_22_05_PROTOCOL_VERSION) {
+		safe_unpackstr_xmalloc(&msg_ptr->env, &uint32_tmp, buffer);
+		safe_unpackstr_xmalloc(&msg_ptr->env_hash, &uint32_tmp, buffer);
+		safe_unpackstr_xmalloc(&msg_ptr->script, &uint32_tmp, buffer);
+		safe_unpackstr_xmalloc(&msg_ptr->script_hash,
+				       &uint32_tmp, buffer);
+	} else
+		  goto unpack_error;
+
+	return SLURM_SUCCESS;
+
+unpack_error:
+	slurmdbd_free_job_heavy_msg(msg_ptr);
+	*msg = NULL;
+	return SLURM_ERROR;
+
+}
+
 static void _pack_job_suspend_msg(dbd_job_suspend_msg_t *msg,
 				  uint16_t rpc_version, buf_t *buffer)
 {
@@ -785,10 +854,7 @@ static void _pack_modify_msg(dbd_modify_msg_t *msg, uint16_t rpc_version,
 		my_rec = slurmdb_pack_federation_rec;
 		break;
 	case DBD_MODIFY_JOB:
-		if (rpc_version >= SLURM_20_02_PROTOCOL_VERSION)
-			my_cond = slurmdb_pack_job_cond;
-		else
-			my_cond = slurmdb_pack_job_modify_cond;
+		my_cond = slurmdb_pack_job_cond;
 		my_rec = slurmdb_pack_job_rec;
 		break;
 	case DBD_MODIFY_QOS:
@@ -839,10 +905,7 @@ static int _unpack_modify_msg(dbd_modify_msg_t **msg, uint16_t rpc_version,
 		my_rec = slurmdb_unpack_federation_rec;
 		break;
 	case DBD_MODIFY_JOB:
-		if (rpc_version >= SLURM_20_02_PROTOCOL_VERSION)
-			my_cond = slurmdb_unpack_job_cond;
-		else
-			my_cond = slurmdb_unpack_job_modify_cond;
+		my_cond = slurmdb_unpack_job_cond;
 		my_rec = slurmdb_unpack_job_rec;
 		break;
 	case DBD_MODIFY_QOS:
@@ -983,7 +1046,7 @@ unpack_error:
 static void _pack_step_complete_msg(dbd_step_comp_msg_t *msg,
 				    uint16_t rpc_version, buf_t *buffer)
 {
-	if (rpc_version >= SLURM_20_11_PROTOCOL_VERSION) {
+	if (rpc_version >= SLURM_MIN_PROTOCOL_VERSION) {
 		pack32(msg->assoc_id, buffer);
 		pack64(msg->db_index, buffer);
 		pack_time(msg->end_time, buffer);
@@ -997,21 +1060,6 @@ static void _pack_step_complete_msg(dbd_step_comp_msg_t *msg,
 		pack16(msg->state, buffer);
 		pack_step_id(&msg->step_id, buffer, rpc_version);
 		pack32(msg->total_tasks, buffer);
-	} else if (rpc_version >= SLURM_MIN_PROTOCOL_VERSION) {
-		pack32(msg->assoc_id, buffer);
-		pack64(msg->db_index, buffer);
-		pack_time(msg->end_time, buffer);
-		pack32(msg->exit_code, buffer);
-		jobacctinfo_pack((struct jobacctinfo *)msg->jobacct,
-				 rpc_version, PROTOCOL_TYPE_DBD, buffer);
-		pack32(msg->step_id.job_id, buffer);
-		pack_time(msg->job_submit_time, buffer);
-		packstr(msg->job_tres_alloc_str, buffer);
-		pack32(msg->req_uid, buffer);
-		pack_time(msg->start_time, buffer);
-		pack16(msg->state, buffer);
-		pack_old_step_id(msg->step_id.step_id, buffer);
-		pack32(msg->total_tasks, buffer);
 	}
 }
 
@@ -1022,7 +1070,7 @@ static int _unpack_step_complete_msg(dbd_step_comp_msg_t **msg,
 	dbd_step_comp_msg_t *msg_ptr = xmalloc(sizeof(dbd_step_comp_msg_t));
 	*msg = msg_ptr;
 
-	if (rpc_version >= SLURM_20_11_PROTOCOL_VERSION) {
+	if (rpc_version >= SLURM_MIN_PROTOCOL_VERSION) {
 		safe_unpack32(&msg_ptr->assoc_id, buffer);
 		safe_unpack64(&msg_ptr->db_index, buffer);
 		safe_unpack_time(&msg_ptr->end_time, buffer);
@@ -1039,24 +1087,6 @@ static int _unpack_step_complete_msg(dbd_step_comp_msg_t **msg,
 					   rpc_version) != SLURM_SUCCESS)
 			goto unpack_error;
 		safe_unpack32(&msg_ptr->total_tasks, buffer);
-	} else if (rpc_version >= SLURM_MIN_PROTOCOL_VERSION) {
-		safe_unpack32(&msg_ptr->assoc_id, buffer);
-		safe_unpack64(&msg_ptr->db_index, buffer);
-		safe_unpack_time(&msg_ptr->end_time, buffer);
-		safe_unpack32(&msg_ptr->exit_code, buffer);
-		jobacctinfo_unpack((struct jobacctinfo **)&msg_ptr->jobacct,
-				   rpc_version, PROTOCOL_TYPE_DBD, buffer, 1);
-		safe_unpack32(&msg_ptr->step_id.job_id, buffer);
-		safe_unpack_time(&msg_ptr->job_submit_time, buffer);
-		safe_unpackstr_xmalloc(&msg_ptr->job_tres_alloc_str,
-				       &uint32_tmp, buffer);
-		safe_unpack32(&msg_ptr->req_uid, buffer);
-		safe_unpack_time(&msg_ptr->start_time, buffer);
-		safe_unpack16(&msg_ptr->state, buffer);
-		safe_unpack32(&msg_ptr->step_id.step_id, buffer);
-		convert_old_step_id(&msg_ptr->step_id.step_id);
-		msg_ptr->step_id.step_het_comp = NO_VAL;
-		safe_unpack32(&msg_ptr->total_tasks, buffer);
 	} else
 		goto unpack_error;
 
@@ -1075,6 +1105,15 @@ static void _pack_step_start_msg(dbd_step_start_msg_t *msg,
 				 uint16_t rpc_version,
 				 buf_t *buffer)
 {
+	/*
+	 * Generate node_inx outside of locks -- not
+	 * jobacct_storage_p_step_start().
+	 * See slurmdbd/acct_storage_p_node_inx() for need to
+	 * use_create_node_inx().
+	 */
+	xassert(!msg->node_inx);
+	msg->node_inx = acct_storage_g_node_inx(NULL, msg->nodes);
+
 	if (rpc_version >= SLURM_21_08_PROTOCOL_VERSION) {
 		pack32(msg->assoc_id, buffer);
 		pack64(msg->db_index, buffer);
@@ -1093,7 +1132,7 @@ static void _pack_step_start_msg(dbd_step_start_msg_t *msg,
 		pack32(msg->task_dist, buffer);
 		pack32(msg->total_tasks, buffer);
 		packstr(msg->tres_alloc_str, buffer);
-	} else if (rpc_version >= SLURM_20_11_PROTOCOL_VERSION) {
+	} else if (rpc_version >= SLURM_MIN_PROTOCOL_VERSION) {
 		pack32(msg->assoc_id, buffer);
 		pack64(msg->db_index, buffer);
 		packstr(msg->name, buffer);
@@ -1109,24 +1148,8 @@ static void _pack_step_start_msg(dbd_step_start_msg_t *msg,
 		pack32(msg->task_dist, buffer);
 		pack32(msg->total_tasks, buffer);
 		packstr(msg->tres_alloc_str, buffer);
-	} else if (rpc_version >= SLURM_MIN_PROTOCOL_VERSION) {
-		pack32(msg->assoc_id, buffer);
-		pack64(msg->db_index, buffer);
-		pack32(msg->step_id.job_id, buffer);
-		packstr(msg->name, buffer);
-		packstr(msg->nodes, buffer);
-		packstr(msg->node_inx, buffer);
-		pack32(msg->node_cnt, buffer);
-		pack_time(msg->start_time, buffer);
-		pack_time(msg->job_submit_time, buffer);
-		pack32(msg->req_cpufreq_min, buffer);
-		pack32(msg->req_cpufreq_max, buffer);
-		pack32(msg->req_cpufreq_gov, buffer);
-		pack_old_step_id(msg->step_id.step_id, buffer);
-		pack32(msg->task_dist, buffer);
-		pack32(msg->total_tasks, buffer);
-		packstr(msg->tres_alloc_str, buffer);
 	}
+	xfree(msg->node_inx);
 }
 
 static int _unpack_step_start_msg(dbd_step_start_msg_t **msg,
@@ -1158,7 +1181,7 @@ static int _unpack_step_start_msg(dbd_step_start_msg_t **msg,
 		safe_unpack32(&msg_ptr->total_tasks, buffer);
 		safe_unpackstr_xmalloc(&msg_ptr->tres_alloc_str,
 				       &uint32_tmp, buffer);
-	} else if (rpc_version >= SLURM_20_11_PROTOCOL_VERSION) {
+	} else if (rpc_version >= SLURM_MIN_PROTOCOL_VERSION) {
 		safe_unpack32(&msg_ptr->assoc_id, buffer);
 		safe_unpack64(&msg_ptr->db_index, buffer);
 		safe_unpackstr_xmalloc(&msg_ptr->name, &uint32_tmp, buffer);
@@ -1177,26 +1200,6 @@ static int _unpack_step_start_msg(dbd_step_start_msg_t **msg,
 		safe_unpack32(&msg_ptr->total_tasks, buffer);
 		safe_unpackstr_xmalloc(&msg_ptr->tres_alloc_str,
 				       &uint32_tmp, buffer);
-	} else if (rpc_version >= SLURM_MIN_PROTOCOL_VERSION) {
-		safe_unpack32(&msg_ptr->assoc_id, buffer);
-		safe_unpack64(&msg_ptr->db_index, buffer);
-		safe_unpack32(&msg_ptr->step_id.job_id, buffer);
-		safe_unpackstr_xmalloc(&msg_ptr->name, &uint32_tmp, buffer);
-		safe_unpackstr_xmalloc(&msg_ptr->nodes, &uint32_tmp, buffer);
-		safe_unpackstr_xmalloc(&msg_ptr->node_inx, &uint32_tmp, buffer);
-		safe_unpack32(&msg_ptr->node_cnt, buffer);
-		safe_unpack_time(&msg_ptr->start_time, buffer);
-		safe_unpack_time(&msg_ptr->job_submit_time, buffer);
-		safe_unpack32(&msg_ptr->req_cpufreq_min, buffer);
-		safe_unpack32(&msg_ptr->req_cpufreq_max, buffer);
-		safe_unpack32(&msg_ptr->req_cpufreq_gov, buffer);
-		safe_unpack32(&msg_ptr->step_id.step_id, buffer);
-		convert_old_step_id(&msg_ptr->step_id.step_id);
-		msg_ptr->step_id.step_het_comp = NO_VAL;
-		safe_unpack32(&msg_ptr->task_dist, buffer);
-		safe_unpack32(&msg_ptr->total_tasks, buffer);
-		safe_unpackstr_xmalloc(&msg_ptr->tres_alloc_str,
-				       &uint32_tmp, buffer);
 	} else
 		goto unpack_error;
 
@@ -1244,7 +1247,12 @@ extern void slurmdbd_pack_id_rc_msg(void *in, uint16_t rpc_version,
 {
 	dbd_id_rc_msg_t *msg = (dbd_id_rc_msg_t *)in;
 
-	if (rpc_version >= SLURM_MIN_PROTOCOL_VERSION) {
+	if (rpc_version >= SLURM_22_05_PROTOCOL_VERSION) {
+		pack32(msg->job_id, buffer);
+		pack64(msg->db_index, buffer);
+		pack64(msg->flags, buffer);
+		pack32(msg->return_code, buffer);
+	} else if (rpc_version >= SLURM_MIN_PROTOCOL_VERSION) {
 		pack32(msg->job_id, buffer);
 		pack64(msg->db_index, buffer);
 		pack32(msg->return_code, buffer);
@@ -1258,7 +1266,12 @@ extern int slurmdbd_unpack_id_rc_msg(void **msg, uint16_t rpc_version,
 
 	*msg = msg_ptr;
 
-	if (rpc_version >= SLURM_MIN_PROTOCOL_VERSION) {
+	if (rpc_version >= SLURM_22_05_PROTOCOL_VERSION) {
+		safe_unpack32(&msg_ptr->job_id, buffer);
+		safe_unpack64(&msg_ptr->db_index, buffer);
+		safe_unpack64(&msg_ptr->flags, buffer);
+		safe_unpack32(&msg_ptr->return_code, buffer);
+	} else if (rpc_version >= SLURM_MIN_PROTOCOL_VERSION) {
 		safe_unpack32(&msg_ptr->job_id, buffer);
 		safe_unpack64(&msg_ptr->db_index, buffer);
 		safe_unpack32(&msg_ptr->return_code, buffer);
@@ -1439,6 +1452,9 @@ extern void slurmdbd_pack_list_msg(dbd_list_msg_t *msg, uint16_t rpc_version,
 	case DBD_GOT_MULT_JOB_START:
 		my_function = slurmdbd_pack_id_rc_msg;
 		break;
+	case DBD_JOB_HEAVY:
+		my_function = _pack_job_heavy_msg;
+		break;
 	case DBD_SEND_MULT_MSG:
 	case DBD_GOT_MULT_MSG:
 		my_function = _pack_buffer;
@@ -1542,6 +1558,10 @@ extern int slurmdbd_unpack_list_msg(dbd_list_msg_t **msg, uint16_t rpc_version,
 		my_function = slurmdbd_unpack_id_rc_msg;
 		my_destroy = slurmdbd_free_id_rc_msg;
 		break;
+	case DBD_JOB_HEAVY:
+		my_function = _unpack_job_heavy_msg;
+		my_destroy = slurmdbd_free_job_heavy_msg;
+		break;
 	case DBD_SEND_MULT_MSG:
 	case DBD_GOT_MULT_MSG:
 		my_function = _unpack_buffer;
@@ -1687,6 +1707,9 @@ extern buf_t *pack_slurmdbd_msg(persist_msg_t *req, uint16_t rpc_version)
 	case DBD_JOB_START:
 		_pack_job_start_msg(req->data, rpc_version, buffer);
 		break;
+	case DBD_JOB_HEAVY:
+		_pack_job_heavy_msg(req->data, rpc_version, buffer);
+		break;
 	case DBD_ID_RC:
 		slurmdbd_pack_id_rc_msg(req->data, rpc_version, buffer);
 		break;
@@ -1890,6 +1913,9 @@ extern int unpack_slurmdbd_msg(persist_msg_t *resp, uint16_t rpc_version,
 		rc = _unpack_job_start_msg(
 			&resp->data, rpc_version, buffer);
 		break;
+	case DBD_JOB_HEAVY:
+		rc = _unpack_job_heavy_msg(&resp->data, rpc_version, buffer);
+		break;
 	case DBD_ID_RC:
 		rc = slurmdbd_unpack_id_rc_msg(
 			&resp->data, rpc_version, buffer);
