@@ -43,8 +43,8 @@
 
 #include "src/common/data.h"
 #include "src/common/log.h"
-#include "src/common/node_select.h"
 #include "src/common/read_config.h"
+#include "src/common/select.h"
 #include "src/common/slurm_protocol_api.h"
 #include "src/common/slurmdbd_defs.h"
 #include "src/common/uid.h"
@@ -52,7 +52,7 @@
 #include "src/common/xmalloc.h"
 #include "src/common/xstring.h"
 
-#include "src/plugins/openapi/dbv0.0.36/api.h"
+#include "src/plugins/openapi/dbv0.0.38/api.h"
 
 /*
  * WARNING: parser uses a ton of macros to avoid massive amounts of copy and
@@ -150,19 +150,13 @@ typedef struct {
 		.required = req,                                              \
 		.type = PARSE_##mtype,                                        \
 	}
-/* only use add_parser_qos_preempt() with slurmdb_qos_rec_t */
-#define add_parser_qos_preempt(req, path)                                     \
-	{                                                                     \
-		.key = path,                                                  \
-		.required = req,                                              \
-		.type = PARSE_QOS_PREEMPT_LIST,                               \
-	}
+
 /*
  * flags are a special exception that just passed the flags_array as the
  * field_offset since the array actually has all of the flag offsets in the
  * struct
  */
-#define add_parser_flags(flags_array, stype, req, field, path)                \
+#define add_parser_flags(flags_array, req, path)                              \
 	{                                                                     \
 		.field_offset = (uintptr_t)flags_array,                       \
 		.key = path,                                                  \
@@ -222,8 +216,7 @@ static const parser_t parse_assoc[] = {
 	/* skipping bf_usage (not packed) */
 	_add_parse(STRING, cluster, "cluster"),
 	_add_parse(QOS_ID, def_qos_id, "default/qos"),
-	add_parser_flags(parser_assoc_flags, slurmdb_assoc_rec_t, false, flags,
-			 "flags"),
+	add_parser_flags(parser_assoc_flags, false, "flags"),
 	/* skip lft */
 	_add_parse(UINT32, grp_jobs, "max/jobs/per/count"),
 	_add_parse(UINT32, grp_jobs_accrue, "max/jobs/per/accruing"),
@@ -231,12 +224,24 @@ static const parser_t parse_assoc[] = {
 	_add_parse(TRES_LIST, grp_tres, "max/tres/total"),
 	/* skipping gres_tres_ctld  (not packed) */
 	_add_parse(TRES_LIST, max_tres_mins_pj, "max/tres/minutes/per/job"),
+	_add_parse(TRES_LIST, grp_tres_mins, "max/tres/group/minutes"),
+	/* skipping grp_tres_mins_ctld (not packed) */
+	_add_parse(TRES_LIST, grp_tres_run_mins, "max/tres/group/active"),
+	/* skipping grp_tres_run_mins_ctld (not packed) */
 	/* skipping max_tres_mins_ctld */
+	/* skipping id */
+	_add_parse(UINT16, is_def, "is_default"),
+	/* skipping lft */
+	_add_parse(UINT32, max_jobs, "max/jobs/active"),
+	_add_parse(UINT32, max_jobs_accrue, "max/jobs/accruing"),
+	_add_parse(UINT32, max_submit_jobs, "max/jobs/total"),
+	/* skipping max_tres_mins_ctld (not packed) */
 	_add_parse(TRES_LIST, max_tres_run_mins, "max/tres/minutes/total"),
 	/* skipping grp_tres_run_mins_ctld (not packed) */
 	_add_parse(UINT32, grp_wall, "max/per/account/wall_clock"),
+	/* skipping max_tres_mins_ctld (not packed) */
 	_add_parse(TRES_LIST, max_tres_pj, "max/tres/per/job"),
-	/* skipping max_tres_ctld */
+	/* skipping max_tres_ctld (not packed) */
 	_add_parse(TRES_LIST, max_tres_pn, "max/tres/per/node"),
 	/* skipping max_tres_pn_ctld */
 	_add_parse(UINT32, max_wall_pj, "max/jobs/per/wall_clock"),
@@ -245,7 +250,7 @@ static const parser_t parse_assoc[] = {
 	/* skip parent_id */
 	_add_parse(STRING, partition, "partition"),
 	_add_parse(UINT32, priority, "priority"),
-	_add_parse(QOS_STR_LIST, qos_list, "qos"),
+	_add_parse(QOS_ID_LIST, qos_list, "qos"),
 	/* skip rgt */
 	_add_parse(UINT32, shares_raw, "shares_raw"),
 	/* slurmdbd should never set uid - it should always be zero */
@@ -260,6 +265,14 @@ static const parser_t parse_assoc[] = {
 	add_parser(slurmdb_user_rec_t, mtype, false, field, path)
 #define _add_parse_req(mtype, field, path) \
 	add_parser(slurmdb_user_rec_t, mtype, true, field, path)
+
+#define _add_flag(flagn, flagv) \
+	add_parser_enum_flag(slurmdb_user_rec_t, flags, flagn, flagv)
+static const parser_enum_t parser_user_flags[] = {
+	_add_flag(SLURMDB_USER_FLAG_DELETED, "DELETED"),
+};
+#undef _add_flag
+
 /* should mirror the structure of slurmdb_user_rec */
 static const parser_t parse_user[] = {
 	_add_parse(ADMIN_LVL, admin_level, "administrator_level"),
@@ -267,6 +280,7 @@ static const parser_t parse_user[] = {
 	_add_parse(COORD_LIST, coord_accts, "coordinators"),
 	_add_parse(STRING, default_acct, "default/account"),
 	_add_parse(STRING, default_wckey, "default/wckey"),
+	add_parser_flags(parser_user_flags, false, "flags"),
 	_add_parse_req(STRING, name, "name"),
 	/* skipping old_name */
 	/* skipping uid (should always be 0) */
@@ -301,6 +315,7 @@ static const parser_t parse_job[] = {
 	/* skip blockid (deprecated bluegene) */
 	_add_parse(STRING, cluster, "cluster"),
 	_add_parse(STRING, constraints, "constraints"),
+	_add_parse(STRING, container, "container"),
 	/* skip db_index */
 	_add_parse(JOB_EXIT_CODE, derived_ec, "derived_exit_code"),
 	_add_parse(STRING, derived_es, "comment/job"),
@@ -309,8 +324,7 @@ static const parser_t parse_job[] = {
 	_add_parse(UINT32, eligible, "time/eligible"),
 	_add_parse(UINT32, end, "time/end"),
 	_add_parse(JOB_EXIT_CODE, exitcode, "exit_code"),
-	add_parser_flags(parser_job_flags, slurmdb_job_rec_t, false, flags,
-			 "flags"),
+	add_parser_flags(parser_job_flags, false, "flags"),
 	/* skipping first_step_ptr (already added in steps) */
 	_add_parse(GROUP_ID, gid, "group"),
 	_add_parse(UINT32, het_job_id, "het/job_id"),
@@ -331,7 +345,7 @@ static const parser_t parse_job[] = {
 	/* skipping show_full */
 	_add_parse(UINT32, eligible, "time/start"),
 	_add_parse(JOB_STATE, state, "state/current"),
-	_add_parse(JOB_REASON, state_reason_prev, "state/previous"),
+	_add_parse(JOB_REASON, state_reason_prev, "state/reason"),
 	_add_parse(UINT32, submit, "time/submission"),
 	_add_parse(JOB_STEPS, steps, "steps"),
 	_add_parse(UINT32, suspended, "time/suspended"),
@@ -370,8 +384,7 @@ static const parser_t parse_acct[] = {
 	_add_parse(STRING, description, "description"),
 	_add_parse(STRING, name, "name"),
 	_add_parse(STRING, organization, "organization"),
-	add_parser_flags(parser_acct_flags, slurmdb_account_rec_t, false, flags,
-			 "flags"),
+	add_parser_flags(parser_acct_flags, false, "flags"),
 };
 #undef _add_parse
 
@@ -406,8 +419,7 @@ static const parser_t parse_wckey[] = {
 	_add_parse_req(STRING, name, "name"),
 	_add_parse_req(STRING, user, "user"),
 	/* skipping uid */
-	add_parser_flags(parser_wckey_flags, slurmdb_wckey_rec_t, false, flags,
-			 "flags"),
+	add_parser_flags(parser_wckey_flags, false, "flags"),
 };
 #undef _add_parse
 #undef _add_parse_req
@@ -469,8 +481,7 @@ static const parser_enum_t parser_qos_preempt_flags[] = {
 static const parser_t parse_qos[] = {
 	/* skipping accounting_list */
 	_add_parse(STRING, description, "description"),
-	add_parser_flags(parser_qos_flags, slurmdb_qos_rec_t, false, flags,
-			 "flags"),
+	add_parser_flags(parser_qos_flags, false, "flags"),
 	_add_parse(UINT32, id, "id"),
 	_add_parse(UINT32, grace_time, "limits/grace_time"),
 	_add_parse(UINT32, grp_jobs_accrue, "limits/max/active_jobs/accruing"),
@@ -480,13 +491,17 @@ static const parser_t parse_qos[] = {
 	_add_parse(TRES_LIST, grp_tres_run_mins,
 		   "limits/max/tres/minutes/per/qos"),
 	/* skipping grp_tres_run_mins_ctld (not packed) */
-	_add_parse_req(STRING, name, "name"),
+	_add_parse(STRING, name, "name"),
 	_add_parse(UINT32, grp_wall, "limits/max/wall_clock/per/qos"),
-	_add_parse(UINT32, max_jobs_pa, "limits/max/jobs/per/account"),
-	_add_parse(UINT32, max_jobs_pu, "limits/max/jobs/per/user"),
+	_add_parse(FLOAT64, limit_factor, "limits/factor"),
+	_add_parse(UINT32, max_jobs_pa,
+		   "limits/max/jobs/active_jobs/per/account"),
+	_add_parse(UINT32, max_jobs_pu, "limits/max/jobs/active_jobs/per/user"),
 	_add_parse(UINT32, max_jobs_accrue_pa,
 		   "limits/max/accruing/per/account"),
 	_add_parse(UINT32, max_jobs_accrue_pu, "limits/max/accruing/per/user"),
+	_add_parse(UINT32, max_submit_jobs_pa, "limits/max/jobs/per/account"),
+	_add_parse(UINT32, max_submit_jobs_pu, "limits/max/jobs/per/user"),
 	_add_parse(TRES_LIST, max_tres_mins_pj,
 		   "limits/max/tres/minutes/per/job"),
 	/* skipping max_tres_mins_pj_ctld (not packed) */
@@ -508,10 +523,8 @@ static const parser_t parse_qos[] = {
 	_add_parse(UINT32, min_prio_thresh, "limits/min/priority_threshold"),
 	_add_parse(TRES_LIST, min_tres_pj, "limits/min/tres/per/job"),
 	/* skipping min_tres_pj_ctld (not packed) */
-	add_parser_qos_preempt(false, "preempt/list"),
-	/* skip preempt_list (only for ops) */
-	add_parser_flags(parser_qos_preempt_flags, slurmdb_qos_rec_t, false,
-			 preempt_mode, "preempt/mode"),
+	_add_parse(QOS_PREEMPT_LIST, preempt_list, "preempt/list"),
+	add_parser_flags(parser_qos_preempt_flags, false, "preempt/mode"),
 	_add_parse(UINT32, preempt_exempt_time, "preempt/exempt_time"),
 	_add_parse(UINT32, priority, "priority"),
 	/* skip usage (not packed) */
@@ -548,8 +561,7 @@ static const parser_t parse_job_step[] = {
 	_add_parse(STRING, pid_str, "pid"),
 	_add_parse(UINT32, req_cpufreq_min, "CPU/requested_frequency/min"),
 	_add_parse(UINT32, req_cpufreq_max, "CPU/requested_frequency/max"),
-	add_parser_flags(parse_job_step_cpu_freq_flags, slurmdb_step_rec_t,
-			 false, req_cpufreq_gov, "CPU/governor"),
+	add_parser_flags(parse_job_step_cpu_freq_flags, false, "CPU/governor"),
 	_add_parse(USER_ID, requid, "kill_request_user"),
 	_add_parse(UINT32, start, "time/start"),
 	_add_parse(JOB_STATE, state, "state"),
@@ -648,8 +660,7 @@ static const parser_t parse_cluster_rec[] = {
 	_add_parse(UINT32, control_port, "controller/port"),
 	/* skip dim_size (BG deprecated) */
 	/* skip fed[eration] support */
-	add_parser_flags(parse_cluster_rec_flags, slurmdb_cluster_rec_t, false,
-			 flags, "flags"),
+	add_parser_flags(parse_cluster_rec_flags, false, "flags"),
 	/* skip lock (not packed) */
 	_add_parse(STRING, name, "name"),
 	_add_parse(STRING, nodes, "nodes"),
@@ -799,12 +810,12 @@ static int _parse_to_int64(const parser_t *const parse, void *obj, data_t *str,
 
 	if (data_get_type(str) == DATA_TYPE_NULL)
 		*dst = (double)NO_VAL;
-	else if (data_convert_type(str, DATA_TYPE_FLOAT) == DATA_TYPE_FLOAT)
-		*dst = data_get_float(str);
+	else if (data_convert_type(str, DATA_TYPE_INT_64) == DATA_TYPE_INT_64)
+		*dst = data_get_int(str);
 	else
 		rc = ESLURM_DATA_CONV_FAILED;
 
-	log_flag(DATA, "%s: string %zd rc[%d]=%s", __func__, *dst, rc,
+	log_flag(DATA, "%s: string %"PRId64" rc[%d]=%s", __func__, *dst, rc,
 		 slurm_strerror(rc));
 
 	return rc;
@@ -833,10 +844,17 @@ static int _parse_to_uint16(const parser_t *const parse, void *obj, data_t *str,
 	int rc = SLURM_SUCCESS;
 
 	if (data_get_type(str) == DATA_TYPE_NULL)
-		*dst = 0;
-	else if (data_convert_type(str, DATA_TYPE_INT_64) == DATA_TYPE_INT_64)
-		*dst = data_get_int(str);
-	else
+		*dst = INFINITE16;
+	else if (data_convert_type(str, DATA_TYPE_INT_64) == DATA_TYPE_INT_64) {
+		if (data_get_int(str) == NO_VAL64)
+			*dst = NO_VAL16;
+		else if (data_get_int(str) == INFINITE64)
+			*dst = INFINITE16;
+		else if (0xFFFFFFFFFFFF0000 & data_get_int(str))
+			rc = ESLURM_DATA_CONV_FAILED;
+		else
+			*dst = data_get_int(str);
+	} else
 		rc = ESLURM_DATA_CONV_FAILED;
 
 	log_flag(DATA, "%s: string %hu rc[%d]=%s", __func__, *dst, rc,
@@ -848,7 +866,7 @@ static int _parse_to_uint16(const parser_t *const parse, void *obj, data_t *str,
 static int _dump_to_uint16(const parser_t *const parse, void *obj, data_t *dst,
 			   const parser_env_t *penv)
 {
-	int16_t *src = (((void *)obj) + parse->field_offset);
+	uint16_t *src = (((void *)obj) + parse->field_offset);
 
 	xassert(data_get_type(dst) == DATA_TYPE_NULL);
 
@@ -874,7 +892,7 @@ static int _parse_to_uint64(const parser_t *const parse, void *obj, data_t *str,
 	else
 		rc = ESLURM_DATA_CONV_FAILED;
 
-	log_flag(DATA, "%s: string %zu rc[%d]=%s", __func__, *dst, rc,
+	log_flag(DATA, "%s: string %"PRIu64" rc[%d]=%s", __func__, *dst, rc,
 		 slurm_strerror(rc));
 
 	return rc;
@@ -883,7 +901,7 @@ static int _parse_to_uint64(const parser_t *const parse, void *obj, data_t *str,
 static int _dump_to_uint64(const parser_t *const parse, void *obj, data_t *dst,
 			   const parser_env_t *penv)
 {
-	int64_t *src = (((void *)obj) + parse->field_offset);
+	uint64_t *src = (((void *)obj) + parse->field_offset);
 
 	xassert(data_get_type(dst) == DATA_TYPE_NULL);
 
@@ -903,12 +921,15 @@ static int _parse_to_uint32(const parser_t *const parse, void *obj, data_t *str,
 	int rc = SLURM_SUCCESS;
 
 	if (data_get_type(str) == DATA_TYPE_NULL) {
-		*dst = 0;
+		*dst = INFINITE;
 	} else if (data_convert_type(str, DATA_TYPE_INT_64) ==
 		   DATA_TYPE_INT_64) {
-		/* catch -1 and set to NO_VAL instead of rolling */
-		if (0xFFFFFFFF00000000 & data_get_int(str))
+		if (data_get_int(str) == NO_VAL64)
 			*dst = NO_VAL;
+		else if (data_get_int(str) == INFINITE64)
+			*dst = INFINITE;
+		else if (0xFFFFFFFF00000000 & data_get_int(str))
+			rc = ESLURM_DATA_CONV_FAILED;
 		else
 			*dst = data_get_int(str);
 	} else
@@ -947,6 +968,7 @@ typedef struct {
 static data_for_each_cmd_t _for_each_parse_flag(data_t *data, void *arg)
 {
 	for_each_parse_flag_t *args = arg;
+	bool found_a_match = false;
 
 	xassert(args->magic == MAGIC_FOREACH_PARSE_FLAGS);
 
@@ -957,6 +979,9 @@ static data_for_each_cmd_t _for_each_parse_flag(data_t *data, void *arg)
 	     f->type; f++) {
 		bool match = !xstrcasecmp(data_get_string(data), f->string);
 
+		if (match)
+			found_a_match = true;
+
 		if (f->type == PARSER_ENUM_FLAG_BIT) {
 			const size_t b = f->size;
 			if (!match)
@@ -1018,6 +1043,12 @@ static data_for_each_cmd_t _for_each_parse_flag(data_t *data, void *arg)
 			fatal("%s: unexpect type", __func__);
 	}
 
+	if (!found_a_match) {
+		resp_error(args->errors, ESLURM_REST_FAIL_PARSING,
+			   "Unknown flag", args->parse->key);
+		return DATA_FOR_EACH_FAIL;
+	}
+
 	return DATA_FOR_EACH_CONT;
 }
 
@@ -1124,24 +1155,43 @@ typedef struct {
 	data_t *errors;
 } for_each_parse_qos_t;
 
+static data_for_each_cmd_t _parse_qos_common(data_t *data,
+					     data_t **name)
+{
+	xassert(name);
+
+	switch (data_get_type(data)) {
+	case DATA_TYPE_STRING:
+		*name = data;
+		break;
+	case DATA_TYPE_DICT:
+		/*
+		 * Note: we ignore everything but name for loading QOS into an
+		 * qos_list as that is the only field accepted
+		 */
+		if (!(*name = data_key_get(data, "name")) ||
+		    data_convert_type(*name, DATA_TYPE_STRING) !=
+			    DATA_TYPE_STRING)
+			return DATA_FOR_EACH_FAIL;
+		break;
+	default:
+		return DATA_FOR_EACH_FAIL;
+	}
+
+	return DATA_FOR_EACH_CONT;
+}
+
 static data_for_each_cmd_t _for_each_parse_qos(data_t *data, void *arg)
 {
+	data_for_each_cmd_t rc;
 	for_each_parse_qos_t *args = arg;
 	data_t *name;
 
 	xassert(args->magic == MAGIC_FOREACH_PARSE_QOS);
 
-	if (data_get_type(data) != DATA_TYPE_DICT)
-		return DATA_FOR_EACH_FAIL;
-
-	/*
-	 * Note: we ignore everything but name for loading QOS into an
-	 * qos_list as that is the only field accepted
-	 */
-
-	if (!(name = data_key_get(data, "name")) ||
-	    data_convert_type(name, DATA_TYPE_STRING) != DATA_TYPE_STRING)
-		return DATA_FOR_EACH_FAIL;
+	rc = _parse_qos_common(data, &name);
+	if (rc != DATA_FOR_EACH_CONT)
+		return rc;
 
 	(void)list_append(args->qos_list, xstrdup(data_get_string(name)));
 
@@ -1256,65 +1306,88 @@ static int _dump_qos_str_list(const parser_t *const parse, void *obj,
 		.magic = MAGIC_FOREACH_DUMP_QOS_STR_LIST,
 		.qos = dst,
 	};
+	/* Convert list of QOS id strings into actual name strings */
+	List qos_list_names = get_qos_name_list(penv->g_qos_list, *qos_list);
+	if (!qos_list_names)
+		return SLURM_SUCCESS;
+
+	list_sort(qos_list_names, slurm_sort_char_list_asc);
 
 	xassert(data_get_type(dst) == DATA_TYPE_NULL);
 	data_set_list(dst);
 
-	if (list_for_each(*qos_list, _foreach_dump_qos_str_list, &args) < 0)
+	if (list_for_each(qos_list_names, _foreach_dump_qos_str_list,
+			  &args) < 0) {
+		FREE_NULL_LIST(qos_list_names);
 		return ESLURM_DATA_CONV_FAILED;
+	}
 
+	FREE_NULL_LIST(qos_list_names);
 	return SLURM_SUCCESS;
 }
 
-#define MAGIC_FOREACH_QOS_PREEMPT_LIST 0xa8eb1313
+#define MAGIC_FOREACH_PARSE_QOS_ID 0xabaa2c19
 typedef struct {
 	int magic;
-	data_t *errors;
 	List qos_list;
-	const parser_env_t *penv;
-} foreach_parse_qos_preempt_list_t;
+	List g_qos_list;
+	data_t *errors;
+} for_each_parse_qos_id_t;
 
-static data_for_each_cmd_t _foreach_parse_qos_preempt_list(data_t *data,
-							   void *arg)
+static data_for_each_cmd_t _for_each_parse_qos_id(data_t *data, void *arg)
 {
-	foreach_parse_qos_preempt_list_t *args = arg;
+	data_for_each_cmd_t rc;
+	for_each_parse_qos_id_t *args = arg;
+	data_t *name;
 
-	xassert(args->magic == MAGIC_FOREACH_QOS_PREEMPT_LIST);
+	xassert(args->magic == MAGIC_FOREACH_PARSE_QOS_ID);
+
+	rc = _parse_qos_common(data, &name);
+	if (rc != DATA_FOR_EACH_CONT)
+		return rc;
+
+	if (slurmdb_addto_qos_char_list(args->qos_list, args->g_qos_list,
+					data_get_string(name), 0) > 0) {
+		return DATA_FOR_EACH_CONT;
+	} else {
+		resp_error(args->errors, ESLURM_REST_FAIL_PARSING,
+			   "QOS name to ID conversion failed",
+			   data_get_string(name));
 
-	if (data_convert_type(data, DATA_TYPE_STRING) != DATA_TYPE_STRING)
 		return DATA_FOR_EACH_FAIL;
-
-	list_append(args->qos_list, xstrdup(data_get_string(data)));
-	return DATA_FOR_EACH_CONT;
+	}
 }
 
-static int _parse_qos_preempt_list(const parser_t *const parse, void *obj,
-				   data_t *src, data_t *errors,
-				   const parser_env_t *penv)
+#define EMPTY_QOS_ID_ENTRY "\'\'"
+static int _parse_qos_id_list(const parser_t *const parse, void *obj,
+			      data_t *src, data_t *errors,
+			      const parser_env_t *penv)
 {
-#ifndef NDEBUG
-	bitstr_t **preempt_bitstr =
-		(((void *)obj) + offsetof(slurmdb_qos_rec_t, preempt_bitstr));
-#endif
-	List *preempt_list =
-		(((void *)obj) + offsetof(slurmdb_qos_rec_t, preempt_list));
-	foreach_parse_qos_preempt_list_t args = {
-		.magic = MAGIC_FOREACH_QOS_PREEMPT_LIST,
-		.penv = penv,
-		.qos_list = list_create(xfree_ptr),
+	List *qos_list = (((void *)obj) + parse->field_offset);
+	for_each_parse_qos_id_t args = {
+		.magic = MAGIC_FOREACH_PARSE_QOS_ID,
+		.errors = errors,
+		.g_qos_list = penv->g_qos_list,
 	};
 
-	xassert(!parse->field_offset);
-	xassert(!*preempt_bitstr);
+	xassert(penv->g_qos_list);
 
-	if ((data_get_type(src) != DATA_TYPE_LIST) ||
-	    (data_list_for_each(src, _foreach_parse_qos_preempt_list, &args) <
-	     0)) {
-		FREE_NULL_LIST(args.qos_list);
+	if (!*qos_list)
+		*qos_list = list_create(xfree_ptr);
+
+	args.qos_list = *qos_list;
+
+	if (data_list_for_each(src, _for_each_parse_qos_id, &args) < 0)
 		return ESLURM_REST_FAIL_PARSING;
-	}
 
-	*preempt_list = args.qos_list;
+	if (list_is_empty(*qos_list)) {
+		/*
+		 * If the QOS list is empty, then we need to set this special
+		 * entry to notify slurmdbd that this is explicilty empty and
+		 * not a no change request
+		 */
+		list_append(*qos_list, EMPTY_QOS_ID_ENTRY);
+	}
 
 	return SLURM_SUCCESS;
 }
@@ -1330,7 +1403,6 @@ static int _dump_qos_preempt_list(const parser_t *const parse, void *obj,
 		(((void *)obj) + offsetof(slurmdb_qos_rec_t, preempt_list));
 #endif
 
-	xassert(!parse->field_offset);
 	xassert(!*preempt_list);
 	xassert(penv->g_qos_list);
 	xassert(data_get_type(dst) == DATA_TYPE_NULL);
@@ -1715,8 +1787,14 @@ static int _dump_assoc_id(const parser_t *const parse, void *obj, data_t *dst,
 
 	if (!(assoc = list_find_first(penv->g_assoc_list,
 				      slurmdb_find_assoc_in_list, associd))) {
-		error("%s: unable to find assocation id#%u",
-		      __func__, *associd);
+		debug("%s: unable to resolve assoc_id %u", __func__, *associd);
+
+		data_set_dict(dst);
+		data_set_int(data_key_set(dst, "id"), *associd);
+		data_set_null(data_key_set(dst, "account"));
+		data_set_null(data_key_set(dst, "cluster"));
+		data_set_null(data_key_set(dst, "partition"));
+		data_set_null(data_key_set(dst, "user"));
 		return SLURM_SUCCESS;
 	}
 
@@ -1773,7 +1851,16 @@ static int _find_tres_id(void *x, void *key)
 
 	xassert(args->magic == MAGIC_FIND_TRES);
 
-	if ((args->tres->id > 0) && args->tres->id == tres->id)
+	debug5("Comparing database tres(name:%s, type:%s, id:%u) with requested(name:%s, type:%s, id:%u).",
+	       tres->name, tres->type, tres->id,
+	       args->tres->name, args->tres->type, args->tres->id);
+
+	if ((args->tres->id > 0) &&
+	    ((args->tres->id == tres->id) &&
+	     (!args->tres->type ||
+	      !xstrcasecmp(args->tres->type, tres->type)) &&
+	     (!args->tres->name ||
+	      !xstrcasecmp(args->tres->name, tres->name))))
 		return 1;
 	if ((!args->tres->name || !args->tres->name[0]) &&
 	    !xstrcasecmp(args->tres->type, tres->type))
@@ -1818,13 +1905,26 @@ static data_for_each_cmd_t _for_each_parse_tres_count(data_t *data, void *arg)
 	if ((ftres = list_find_first(args->penv->g_tres_list, _find_tres_id,
 				     &targs))) {
 		if ((tres->id > 0) && tres->id != ftres->id) {
-			resp_error(errors, ESLURM_INVALID_TRES,
-				   "TRES id unknown", "id");
+			char *msg = NULL;
+			xstrfmtcat(msg,
+				   "Requested TRES id(%d) doesn't match TRES type/name(%s/%s) which id is %d",
+				   tres->id, ftres->type, ftres->name,
+				   ftres->id);
+			resp_error(errors, ESLURM_INVALID_TRES, msg, __func__);
+			xfree(msg);
 			return DATA_FOR_EACH_FAIL;
 		}
 
 		if (!tres->id)
 			tres->id = ftres->id;
+	} else {
+		char *msg = NULL;
+		xstrfmtcat(msg,
+			   "Couldn't find TRES matching name:%s type:%s",
+			   targs.tres->name, targs.tres->type);
+		resp_error(errors, ESLURM_INVALID_TRES, msg, __func__);
+		xfree(msg);
+		return DATA_FOR_EACH_FAIL;
 	}
 
 	return DATA_FOR_EACH_CONT;
@@ -1834,6 +1934,7 @@ static int _parse_tres_list(const parser_t *const parse, void *obj, data_t *src,
 			    data_t *errors, const parser_env_t *penv)
 {
 	char **tres = (((void *)obj) + parse->field_offset);
+	int rc;
 	for_each_parse_tres_t args = {
 		.magic = MAGIC_FOREACH_PARSE_TRES_COUNT,
 		.penv = penv,
@@ -1843,19 +1944,28 @@ static int _parse_tres_list(const parser_t *const parse, void *obj, data_t *src,
 
 	if (!penv->g_tres_list) {
 		xassert(penv->g_tres_list);
-		return ESLURM_NOT_SUPPORTED;
+		rc = ESLURM_NOT_SUPPORTED;
+		goto cleanup;
 	}
 
-	if (data_get_type(src) != DATA_TYPE_LIST)
-		return ESLURM_REST_FAIL_PARSING;
+	if (data_get_type(src) != DATA_TYPE_LIST) {
+		rc = ESLURM_REST_FAIL_PARSING;
+		goto cleanup;
+	}
 
-	if (data_list_for_each(src, _for_each_parse_tres_count, &args) < 0)
-		return ESLURM_REST_FAIL_PARSING;
+	if (data_list_for_each(src, _for_each_parse_tres_count, &args) < 0) {
+		rc = ESLURM_REST_FAIL_PARSING;
+		goto cleanup;
+	}
 
 	if ((*tres = slurmdb_make_tres_string(args.tres, TRES_STR_FLAG_SIMPLE)))
-		return SLURM_SUCCESS;
+		rc = SLURM_SUCCESS;
 	else
-		return ESLURM_REST_FAIL_PARSING;
+		rc = ESLURM_REST_FAIL_PARSING;
+
+cleanup:
+	FREE_NULL_LIST(args.tres);
+	return rc;
 }
 
 #define MAGIC_LIST_PER_TRES 0xf7f8baf0
@@ -2524,7 +2634,7 @@ static int _dump_stats_user_list(const parser_t *const parse, void *obj,
 	xassert(data_get_type(dst) == DATA_TYPE_NULL);
 	args.users = data_set_list(dst);
 
-	if (list_for_each(*user_list, _foreach_stats_user_rpc, &args))
+	if (list_for_each(*user_list, _foreach_stats_user_rpc, &args) < 0)
 		return ESLURM_DATA_CONV_FAILED;
 
 	return SLURM_SUCCESS;
@@ -2802,7 +2912,8 @@ const parser_funcs_t funcs[] = {
 	_add_func(_parse_flags, _dump_flags, PARSE_FLAGS),
 	_add_func(_parse_qos_str_id, _dump_qos_str_id, PARSE_QOS_ID),
 	_add_func(_parse_qos_str_list, _dump_qos_str_list, PARSE_QOS_STR_LIST),
-	_add_func(_parse_qos_preempt_list, _dump_qos_preempt_list,
+	_add_func(_parse_qos_id_list, _dump_qos_str_list, PARSE_QOS_ID_LIST),
+	_add_func(_parse_qos_id_list, _dump_qos_preempt_list,
 		  PARSE_QOS_PREEMPT_LIST),
 	_add_func(_parse_tres, _dump_tres, PARSE_TRES),
 	_add_func(_parse_tres_list, _dump_tres_list, PARSE_TRES_LIST),
@@ -2889,21 +3000,33 @@ static int _parser_run(void *obj, const parser_t *const parse,
 	int rc = SLURM_SUCCESS;
 
 	for (int i = 0; (!rc) && (i < parse_member_count); i++) {
-		for (int f = 0; f < ARRAY_SIZE(funcs); f++) {
-			data_t *pd = data_resolve_dict_path(data, parse[i].key);
+		data_t *pd = data_resolve_dict_path(data, parse[i].key);
+		for (int f = 0; pd && (f < ARRAY_SIZE(funcs)); f++) {
 
-			if (pd && parse[i].type == funcs[f].type) {
+			if (parse[i].type == funcs[f].type) {
 				xassert(funcs[f].rfunc);
 				rc = funcs[f].rfunc((parse + i), obj, pd,
 						    errors, penv);
+				break;
 			}
 		}
 
-		if (rc && parse[i].required)
-			resp_error(errors, rc, "Required field failed to parse",
-				   parse[i].key);
-		else
-			rc = SLURM_SUCCESS;
+		if (!pd && parse[i].required) {
+			char *tmp_str =
+				xstrdup_printf("Missing required field '%s'",
+					       parse[i].key);
+			resp_error(errors, rc, tmp_str, __func__);
+			xfree(tmp_str);
+			break;
+		} else if (rc) {
+			char *tmp_str =
+				xstrdup_printf("Failed to parse %sfield '%s'",
+					       parse[i].required ?
+					       "required " : "",
+					       parse[i].key);
+			resp_error(errors, rc, tmp_str, __func__);
+			xfree(tmp_str);
+		}
 	}
 
 	return rc;
