@@ -39,6 +39,7 @@
 #define _GNU_SOURCE
 
 #include <search.h>
+#include <signal.h>
 #include <stdint.h>
 #include <unistd.h>
 
@@ -48,10 +49,10 @@
 #include "src/common/env.h"
 #include "src/common/list.h"
 #include "src/common/log.h"
-#include "src/common/node_select.h"
 #include "src/common/parse_time.h"
 #include "src/common/proc_args.h"
 #include "src/common/ref.h"
+#include "src/common/select.h"
 #include "src/common/slurm_acct_gather_profile.h"
 #include "src/common/slurm_protocol_api.h"
 #include "src/common/slurm_protocol_defs.h"
@@ -59,13 +60,14 @@
 #include "src/common/strlcpy.h"
 #include "src/common/tres_bind.h"
 #include "src/common/tres_frequency.h"
+#include "src/common/uid.h"
 #include "src/common/xassert.h"
 #include "src/common/xmalloc.h"
 #include "src/common/xstring.h"
 
 #include "src/slurmrestd/operations.h"
 
-#include "src/plugins/openapi/v0.0.36/api.h"
+#include "src/plugins/openapi/v0.0.38/api.h"
 
 typedef struct {
 	const char *param;
@@ -107,9 +109,10 @@ const params_t job_params[] = {
 	{ "constraint", 'C' },
 	{ "constraints", 'C' },
 	{ "contiguous", LONG_OPT_CONTIGUOUS, true },
+	{ "container", LONG_OPT_CONTAINER },
 	{ "core_specification", 'S' },
 	{ "cores_per_socket", LONG_OPT_CORESPERSOCKET },
-	{ "cpu_binding hint", LONG_OPT_HINT },
+	{ "cpu_binding_hint", LONG_OPT_HINT },
 	{ "cpu_binding", LONG_OPT_CPU_BIND, true },
 	{ "cpu_frequency", LONG_OPT_CPU_FREQ },
 	{ "cpus_per_gpu", LONG_OPT_CPUS_PER_GPU },
@@ -149,20 +152,19 @@ const params_t job_params[] = {
 	{ "kill_command", 'K', true },
 	{ "kill_on_bad_exit", 'K', true },
 	{ "kill_on_invalid_dependency", LONG_OPT_KILL_INV_DEP },
-	{ "kill_on_invalid dependency", LONG_OPT_KILL_INV_DEP },
 	{ "label", 'l', true },
 	{ "license", 'L' },
 	{ "licenses", 'L' },
 	{ "mail_type", LONG_OPT_MAIL_TYPE },
 	{ "mail_user", LONG_OPT_MAIL_USER },
 	{ "max_threads", 'T', true },
-	{ "msc_label", LONG_OPT_MCS_LABEL },
+	{ "mcs_label", LONG_OPT_MCS_LABEL },
 	{ "memory_binding", LONG_OPT_MEM_BIND },
-	{ "memory_per_CPU", LONG_OPT_MEM_PER_CPU },
-	{ "memory_per_GPU", LONG_OPT_MEM_PER_GPU },
+	{ "memory_per_cpu", LONG_OPT_MEM_PER_CPU },
+	{ "memory_per_gpu", LONG_OPT_MEM_PER_GPU },
 	{ "memory_per_node", LONG_OPT_MEM },
 	{ "message_timeout", LONG_OPT_MSG_TIMEOUT, true },
-	{ "minimum_CPUs_per_node", LONG_OPT_MINCPUS },
+	{ "minimum_cpus_per_node", LONG_OPT_MINCPUS },
 	{ "minimum_nodes", LONG_OPT_USE_MIN_NODES },
 	{ "mpi", LONG_OPT_MPI, true },
 	{ "multiple_program", LONG_OPT_MULTI, true },
@@ -185,9 +187,10 @@ const params_t job_params[] = {
 	{ "hetjob_group", LONG_OPT_HET_GROUP, true },
 	{ "parsable", LONG_OPT_PARSABLE, true },
 	{ "partition", 'p' },
+	{ "prefer", LONG_OPT_PREFER },
 	{ "power_flags", LONG_OPT_POWER, true },
 	{ "preserve_environment", 'E', true },
-	{ "priority", LONG_OPT_PRIORITY, true },
+	{ "priority", LONG_OPT_PRIORITY, false },
 	{ "profile", LONG_OPT_PROFILE },
 	{ "prolog", LONG_OPT_PROLOG, true },
 	{ "propagate", LONG_OPT_PROPAGATE, true },
@@ -208,9 +211,7 @@ const params_t job_params[] = {
 	{ "sockets_per_node", LONG_OPT_SOCKETSPERNODE },
 	{ "spread_job", LONG_OPT_SPREAD_JOB },
 	{ "standard_error", 'e' },
-	{ "standard_in", 'i' },
 	{ "standard_input", 'i' },
-	{ "standard_out", 'o' },
 	{ "standard_output", 'o' },
 	{ "task_epilog", LONG_OPT_TASK_EPILOG, true },
 	{ "task_prolog", LONG_OPT_TASK_PROLOG, true },
@@ -227,10 +228,10 @@ const params_t job_params[] = {
 	{ "temporary_disk_per_node", LONG_OPT_TMP },
 	{ "test_only", LONG_OPT_TEST_ONLY },
 	{ "thread_specification", LONG_OPT_THREAD_SPEC },
-	{ "threads_per_Core", LONG_OPT_THREADSPERCORE },
+	{ "threads_per_core", LONG_OPT_THREADSPERCORE },
 	{ "threads", 'T', true },
 	{ "time_limit", 't' },
-	{ "time minimum", LONG_OPT_TIME_MIN },
+	{ "time_minimum", LONG_OPT_TIME_MIN },
 	/* Handler for LONG_OPT_TRES_PER_JOB never defined
 	 * { "TRES per job", LONG_OPT_TRES_PER_JOB, true },
 	 */
@@ -270,22 +271,6 @@ static void _list_delete_job_desc_msg_t(void *_p)
 	slurm_free_job_desc_msg(_p);
 }
 
-#define _parse_error(...)                                                    \
-	do {                                                                 \
-		const char *error_string = xstrdup_printf(__VA_ARGS__);      \
-		error("%s", error_string);                                   \
-		data_t *error = data_list_append(errors);                    \
-		data_set_dict(error);                                        \
-		data_set_string(data_key_set(error, "error"), error_string); \
-		xfree(error_string);                                         \
-		if (errno) {                                                 \
-			rc = errno;                                          \
-			errno = 0;                                           \
-		} else                                                       \
-			rc = SLURM_ERROR;                                    \
-		data_set_int(data_key_set(error, "error_code"), rc);         \
-	} while (0)
-
 typedef struct {
 	slurm_opt_t *opt;
 	data_t *errors;
@@ -307,20 +292,22 @@ static data_for_each_cmd_t _per_job_param(const char *key, const data_t *data,
 	xstrtolower(lkey);
 
 	if (!(rc = hsearch_r(e, FIND, &re, &hash_params))) {
-		_parse_error("Unknown key \"%s\": %m", lkey);
+		resp_error(errors, rc, "hsearch_r",
+			   "Unknown key \"%s\"", lkey);
 		return DATA_FOR_EACH_FAIL;
 	}
 
 	p = re->data;
 	if (p->disabled) {
-		_parse_error("Disabled key: \"%s\"", p->param);
+		resp_error(errors, rc, "openapi specification",
+			   "Disabled key: \"%s\"", p->param);
 		return DATA_FOR_EACH_FAIL;
 	}
 
 	if ((rc = slurm_process_option_data(args->opt, p->optval, data,
 					    errors))) {
-		_parse_error("process failed for key %s with error: %s",
-			     key, slurm_strerror(rc));
+		resp_error(errors, rc, "slurm_process_option_data",
+			   "Unable to process key \"%s\"", lkey);
 		return DATA_FOR_EACH_FAIL;
 	}
 
@@ -334,14 +321,10 @@ static data_for_each_cmd_t _per_job_param(const char *key, const data_t *data,
 static int _fill_job_desc_from_opts(slurm_opt_t *opt, job_desc_msg_t *desc)
 {
 	const sbatch_opt_t *sbopt = opt->sbatch_opt;
-	extern char **environ;
 
 	if (!desc)
 		return -1;
 
-	if (!opt->job_name)
-		desc->name = xstrdup("openapi");
-
 	desc->array_inx = xstrdup(sbopt->array_inx);
 	desc->batch_features = sbopt->batch_features;
 	desc->container = xstrdup(opt->container);
@@ -420,14 +403,14 @@ static job_desc_msg_t *_parse_job_desc(const data_t *job, data_t *errors,
 	}
 
 	req = slurm_opt_create_job_desc(&opt, !update_only);
-	if (!update_only)
-		req->task_dist = SLURM_DIST_UNKNOWN;
-
-	if (_fill_job_desc_from_opts(&opt, req)) {
+	if (_fill_job_desc_from_opts(&opt, req) == -1) {
 		rc = SLURM_ERROR;
 		goto cleanup;
 	}
 
+	if (!update_only)
+		req->task_dist = SLURM_DIST_UNKNOWN;
+
 	if (!update_only && (!req->environment || !req->env_size)) {
 		/*
 		 * Jobs provided via data must have their environment
@@ -534,15 +517,10 @@ static data_for_each_cmd_t _parse_job_component(const data_t *data, void *arg)
 
 		list_append(rc->jobs, job_desc);
 	} else { /* parsing failed */
-		data_t *error = data_list_append(j->errors);
-		char *error_string = xstrdup_printf(
-			"%s: unexpected failure parsing het job: %zd",
-			__func__, j->i);
-		data_set_dict(error);
-		data_set_string(data_key_set(error, "error"), error_string);
-		//error("%s", error_string);
-		xfree(error_string);
-		rc->rc = ESLURM_REST_FAIL_PARSING;
+		rc->rc = resp_error(j->errors, ESLURM_REST_FAIL_PARSING,
+				    "_parse_job_desc",
+				    "unexpected failure parsing het job: %zd",
+				    j->i);
 		return DATA_FOR_EACH_FAIL;
 	}
 
@@ -589,6 +567,108 @@ static job_parse_list_t _parse_job_list(const data_t *jobs, char *script,
 	return rc;
 }
 
+static void _dump_node_res(data_t *dnodes, job_resources_t *j,
+			   const size_t node_inx, const char *nodename,
+			   const size_t sock_inx, size_t *bit_inx,
+			   const size_t array_size)
+{
+	size_t bit_reps;
+	data_t *dnode = data_set_dict(data_list_append(dnodes));
+	data_t *dsockets = data_set_dict(data_key_set(dnode, "sockets"));
+	data_t **sockets;
+
+	sockets = xcalloc(j->sockets_per_node[sock_inx], sizeof(*sockets));
+
+	/* per node */
+
+	data_set_string(data_key_set(dnode, "nodename"), nodename);
+
+	data_set_int(data_key_set(dnode, "cpus_used"), j->cpus_used[node_inx]);
+	data_set_int(data_key_set(dnode, "memory_used"),
+		     j->memory_used[node_inx]);
+	data_set_int(data_key_set(dnode, "memory_allocated"),
+		     j->memory_allocated[node_inx]);
+
+	/* set the used cores as found */
+
+	bit_reps = j->sockets_per_node[sock_inx] *
+		   j->cores_per_socket[sock_inx];
+	for (size_t i = 0; i < bit_reps; i++) {
+		size_t socket_inx = i / j->cores_per_socket[sock_inx];
+		size_t core_inx = i % j->cores_per_socket[sock_inx];
+
+		xassert(*bit_inx < array_size);
+
+		if (*bit_inx >= array_size) {
+			error("%s: unexpected invalid bit index:%zu/%zu",
+			      __func__, *bit_inx, array_size);
+			break;
+		}
+
+		if (bit_test(j->core_bitmap, *bit_inx)) {
+			data_t *dcores;
+
+			if (!sockets[socket_inx]) {
+				sockets[socket_inx] = data_set_dict(
+					data_key_set_int(dsockets, socket_inx));
+				dcores = data_set_dict(data_key_set(
+					sockets[socket_inx], "cores"));
+			} else {
+				dcores = data_key_get(sockets[socket_inx],
+						      "cores");
+			}
+
+			if (bit_test(j->core_bitmap_used, *bit_inx)) {
+				data_set_string(data_key_set_int(dcores,
+								 core_inx),
+						"allocated_and_in_use");
+			} else {
+				data_set_string(data_key_set_int(dcores,
+								 core_inx),
+						"allocated");
+			}
+		}
+
+		(*bit_inx)++;
+	}
+
+	xfree(sockets);
+}
+
+/* log_job_resources() used as an example */
+static void _dump_nodes_res(data_t *dnodes, job_resources_t *j)
+{
+	hostlist_t hl = NULL;
+	size_t bit_inx = 0;
+	size_t array_size;
+	size_t sock_inx = 0, sock_reps = 0;
+
+	if (!j->cores_per_socket || !j->nhosts) {
+		/* not enough info present */
+		return;
+	}
+
+	hl = hostlist_create(j->nodes);
+	array_size = bit_size(j->core_bitmap);
+
+	for (size_t node_inx = 0; node_inx < j->nhosts; node_inx++) {
+		char *nodename = hostlist_nth(hl, node_inx);
+
+		if (sock_reps >= j->sock_core_rep_count[sock_inx]) {
+			sock_inx++;
+			sock_reps = 0;
+		}
+		sock_reps++;
+
+		_dump_node_res(dnodes, j, node_inx, nodename, sock_inx,
+			       &bit_inx, array_size);
+
+		free(nodename);
+	}
+
+	FREE_NULL_HOSTLIST(hl);
+}
+
 static data_t *dump_job_info(slurm_job_info_t *job, data_t *jd)
 {
 	xassert(data_get_type(jd) == DATA_TYPE_NULL);
@@ -626,8 +706,6 @@ static data_t *dump_job_info(slurm_job_info_t *job, data_t *jd)
 				"GRES_ENFORCE_BIND");
 	if (job->bitflags & TEST_NOW_ONLY)
 		data_set_string(data_list_append(bitflags), "TEST_NOW_ONLY");
-	if (job->bitflags & NODE_REBOOT)
-		data_set_string(data_list_append(bitflags), "NODE_REBOOT");
 	if (job->bitflags & SPREAD_JOB)
 		data_set_string(data_list_append(bitflags), "SPREAD_JOB");
 	if (job->bitflags & USE_MIN_NODES)
@@ -641,7 +719,7 @@ static data_t *dump_job_info(slurm_job_info_t *job, data_t *jd)
 	if (job->bitflags & HET_JOB_FLAG)
 		data_set_string(data_list_append(bitflags), "HET_JOB_FLAG");
 	if (job->bitflags & JOB_CPUS_SET)
-		data_set_string(data_list_append(bitflags), "JOB_CPUS_SET ");
+		data_set_string(data_list_append(bitflags), "JOB_CPUS_SET");
 	if (job->bitflags & TOP_PRIO_TMP)
 		data_set_string(data_list_append(bitflags), "TOP_PRIO_TMP");
 	if (job->bitflags & JOB_ACCRUE_OVER)
@@ -664,6 +742,7 @@ static data_t *dump_job_info(slurm_job_info_t *job, data_t *jd)
 			job->cluster_features);
 	data_set_string(data_key_set(jd, "command"), job->command);
 	data_set_string(data_key_set(jd, "comment"), job->comment);
+	data_set_string(data_key_set(jd, "container"), job->container);
 	if (job->contiguous != NO_VAL16)
 		data_set_bool(data_key_set(jd, "contiguous"),
 			      job->contiguous == 1);
@@ -741,83 +820,35 @@ static data_t *dump_job_info(slurm_job_info_t *job, data_t *jd)
 				job->gres_detail_str[i]);
 	if (job->group_id == NO_VAL)
 		data_set_null(data_key_set(jd, "group_id"));
-	else
+	else {
 		data_set_int(data_key_set(jd, "group_id"), job->group_id);
+		data_set_string_own(
+			data_key_set(jd, "group_name"),
+			gid_to_string_or_null((gid_t) job->group_id));
+	}
 	if (job->job_id == NO_VAL)
 		data_set_null(data_key_set(jd, "job_id"));
 	else
 		data_set_int(data_key_set(jd, "job_id"), job->job_id);
-	data_t *jrsc = data_key_set(jd, "job_resources");
-	data_set_dict(jrsc);
+
 	if (job->job_resrcs) {
 		/* based on log_job_resources() */
+		data_t *jrsc = data_set_dict(data_key_set(jd, "job_resources"));
 		job_resources_t *j = job->job_resrcs;
 		data_set_string(data_key_set(jrsc, "nodes"), j->nodes);
-		const size_t array_size = bit_size(j->core_bitmap);
-		size_t sock_inx = 0, sock_reps = 0, bit_inx = 0;
 
-		data_set_int(data_key_set(jrsc, "allocated_cpus"), j->ncpus);
+		if (slurm_conf.select_type_param & (CR_CORE|CR_SOCKET))
+			data_set_int(data_key_set(jrsc, "allocated_cores"),
+				     j->ncpus);
+		else if (slurm_conf.select_type_param & CR_CPU)
+			data_set_int(data_key_set(jrsc, "allocated_cpus"),
+				     j->ncpus);
+
 		data_set_int(data_key_set(jrsc, "allocated_hosts"), j->nhosts);
 
-		data_t *nodes = data_key_set(jrsc, "allocated_nodes");
-		data_set_dict(nodes);
-		for (size_t node_inx = 0; node_inx < j->nhosts; node_inx++) {
-			data_t *node = data_key_set_int(nodes, node_inx);
-			data_set_dict(node);
-			data_t *sockets = data_key_set(node, "sockets");
-			data_t *cores = data_key_set(node, "cores");
-			data_set_dict(sockets);
-			data_set_dict(cores);
-			const size_t bit_reps =
-				((size_t) j->sockets_per_node[sock_inx]) *
-				((size_t) j->cores_per_socket[sock_inx]);
-
-			if (sock_reps >= j->sock_core_rep_count[sock_inx]) {
-				sock_inx++;
-				sock_reps = 0;
-			}
-			sock_reps++;
-
-			if (j->memory_allocated)
-				data_set_int(data_key_set(node,
-							  "memory"),
-					     j->memory_allocated[node_inx]);
-
-			data_set_int(data_key_set(node, "cpus"),
-				     j->cpus[node_inx]);
-
-			for (size_t i = 0; i < bit_reps; i++) {
-				if (bit_inx >= array_size) {
-					error("%s: array size wrong", __func__);
-					xassert(false);
-					break;
-				}
-				if (bit_test(j->core_bitmap, bit_inx)) {
-					data_t *socket = data_key_set_int(
-						sockets,
-						(i /
-						 j->cores_per_socket[sock_inx]));
-					data_t *core = data_key_set_int(
-						cores,
-						(i %
-						 j->cores_per_socket[sock_inx]));
-
-					if (bit_test(j->core_bitmap_used,
-						     bit_inx)) {
-						data_set_string(socket,
-								"assigned");
-						data_set_string(core,
-								"assigned");
-					} else {
-						data_set_string(socket,
-								"unassigned");
-						data_set_string(core,
-								"unassigned");
-					}
-				}
-				bit_inx++;
-			}
-		}
+		_dump_nodes_res(
+			data_set_list(data_key_set(jrsc, "allocated_nodes")),
+			j);
 	}
 	data_set_string(data_key_set(jd, "job_state"),
 			job_state_string(job->job_state));
@@ -870,6 +901,7 @@ static data_t *dump_job_info(slurm_job_info_t *job, data_t *jd)
 			job->het_job_id_set);
 	data_set_int(data_key_set(jd, "het_job_offset"), job->het_job_offset);
 	data_set_string(data_key_set(jd, "partition"), job->partition);
+	data_set_string(data_key_set(jd, "prefer"), job->prefer);
 	if (job->pn_min_memory & MEM_PER_CPU) {
 		data_set_null(data_key_set(jd, "memory_per_node"));
 		data_set_int(data_key_set(jd, "memory_per_cpu"),
@@ -976,6 +1008,7 @@ static data_t *dump_job_info(slurm_job_info_t *job, data_t *jd)
 	data_set_int(data_key_set(jd, "suspend_time"), job->suspend_time);
 	data_set_string(data_key_set(jd, "system_comment"),
 			job->system_comment);
+	data_set_string(data_key_set(jd, "container"), job->container);
 	if (job->time_limit != INFINITE)
 		data_set_int(data_key_set(jd, "time_limit"), job->time_limit);
 	else
@@ -1000,7 +1033,13 @@ static data_t *dump_job_info(slurm_job_info_t *job, data_t *jd)
 	data_set_string(data_key_set(jd, "tres_alloc_str"),
 			job->tres_alloc_str);
 	data_set_int(data_key_set(jd, "user_id"), job->user_id);
-	data_set_string(data_key_set(jd, "user_name"), job->user_name);
+	if (job->user_name) {
+		data_set_string(data_key_set(jd, "user_name"), job->user_name);
+	} else {
+		data_set_string_own(
+			data_key_set(jd, "user_name"),
+			uid_to_string_or_null((uid_t) job->user_id));
+	}
 	/* wait4switch intentionally omitted */
 	data_set_string(data_key_set(jd, "wckey"), job->wckey);
 	data_set_string(data_key_set(jd, "current_working_directory"),
@@ -1015,50 +1054,53 @@ static int _op_handler_jobs(const char *context_id,
 {
 	int rc = SLURM_SUCCESS;
 	job_info_msg_t *job_info_ptr = NULL;
-	(void) populate_response_format(resp);
+	data_t *errors = populate_response_format(resp);
 	data_t *jobs = data_set_list(data_key_set(resp, "jobs"));
+	time_t update_time = 0; /* default to unix epoch */
 
 	debug4("%s: jobs handler called by %s", __func__, context_id);
 
-	rc = slurm_load_jobs((time_t)NULL, &job_info_ptr,
-			     SHOW_ALL|SHOW_DETAIL);
+	if ((rc = get_date_param(query, "update_time", &update_time)))
+	    goto done;
 
-	if (rc == SLURM_SUCCESS && job_info_ptr &&
-	    job_info_ptr->record_count) {
+	rc = slurm_load_jobs(update_time, &job_info_ptr,
+			     SHOW_ALL | SHOW_DETAIL);
+
+	if (rc == SLURM_NO_CHANGE_IN_DATA) {
+		/* no-op: nothing to do here */
+	} else if ((rc == SLURM_SUCCESS) && job_info_ptr &&
+		   job_info_ptr->record_count) {
 		for (size_t i = 0; i < job_info_ptr->record_count; ++i) {
 			dump_job_info(job_info_ptr->job_array + i,
 				      data_list_append(jobs));
 		}
+	} else if (rc) {
+		resp_error(errors, rc, "slurm_load_jobs",
+			   "Failed while looking for jobs");
 	}
 
+done:
 	slurm_free_job_info_msg(job_info_ptr);
 
 	return rc;
 }
 
-#define _job_error(...)                                                      \
-	do {                                                                 \
-		const char *error_string = xstrdup_printf(__VA_ARGS__);      \
-		error("%s", error_string);                                   \
-		data_t *error = data_list_append(errors);                    \
-		data_set_dict(error);                                        \
-		data_set_string(data_key_set(error, "error"), error_string); \
-		xfree(error_string);                                         \
-		if (errno) {                                                 \
-			rc = errno;                                          \
-			errno = 0;                                           \
-		} else                                                       \
-			rc = SLURM_ERROR;                                    \
-		data_set_int(data_key_set(error, "error_code"), errno);      \
-	} while (0)
-
 static int _handle_job_get(const char *context_id, http_request_method_t method,
 			   data_t *parameters, data_t *query, int tag,
-			   data_t *resp, const uint32_t job_id,
+			   data_t *resp, const char *job_id_str,
 			   data_t *const errors)
 {
 	int rc = SLURM_SUCCESS;
 	job_info_msg_t *job_info_ptr = NULL;
+	uint32_t job_id = slurm_xlate_job_id((char *) job_id_str);
+
+	if (!job_id) {
+		rc = ESLURM_REST_INVALID_JOBS_DESC;
+		resp_error(errors, rc, "_handle_job_get",
+			   "Unable to find JobId=%s", job_id_str);
+		return rc;
+	}
+
 	rc = slurm_load_job(&job_info_ptr, job_id, SHOW_ALL|SHOW_DETAIL);
 	data_t *jobs = data_set_list(data_key_set(resp, "jobs"));
 
@@ -1068,7 +1110,8 @@ static int _handle_job_get(const char *context_id, http_request_method_t method,
 				      data_list_append(jobs));
 		}
 	} else {
-		_job_error("%s: unknown job %d", __func__, job_id);
+		resp_error(errors, rc, "slurm_load_job",
+			   "Unable to find JobId=%s", job_id_str);
 	}
 
 	slurm_free_job_info_msg(job_info_ptr);
@@ -1079,19 +1122,17 @@ static int _handle_job_get(const char *context_id, http_request_method_t method,
 static int _handle_job_delete(const char *context_id,
 			      http_request_method_t method,
 			      data_t *parameters, data_t *query, int tag,
-			      data_t *resp, const uint32_t job_id,
+			      data_t *resp, const char *job_id,
 			      data_t *const errors, const int signal)
 {
-	if (slurm_kill_job(job_id, signal, KILL_FULL_JOB)) {
-		int rc;
-
+	if (slurm_kill_job2(job_id, signal, KILL_FULL_JOB, NULL)) {
 		/* Already signaled jobs are considered a success here */
 		if (errno == ESLURM_ALREADY_DONE)
 			return SLURM_SUCCESS;
 
-		_job_error("%s: unable to kill job %d with signal %d: %s",
-			   __func__, job_id, signal, slurm_strerror(errno));
-		return rc;
+		return resp_error(errors, errno,"slurm_kill_job",
+				  "unable to kill JobId=%s with signal %d: %s",
+				  job_id, signal, slurm_strerror(errno));
 	}
 
 	return SLURM_SUCCESS;
@@ -1100,52 +1141,54 @@ static int _handle_job_delete(const char *context_id,
 static int _handle_job_post(const char *context_id,
 			    http_request_method_t method, data_t *parameters,
 			    data_t *query, int tag, data_t *resp,
-			    const uint32_t job_id, data_t *const errors)
+			    const char *job_id, data_t *const errors)
 {
 	int rc = SLURM_SUCCESS;
+	job_parse_list_t jobs_rc;
 
 	if (get_log_level() >= LOG_LEVEL_DEBUG5) {
 		char *buffer = NULL;
 
 		data_g_serialize(&buffer, query, MIME_TYPE_JSON,
 				 DATA_SER_FLAGS_COMPACT);
-		debug5("%s: job update from %s: %s", __func__, context_id,
-		       buffer);
-
+		debug5("%s: job update from %s: %s",
+		       __func__, context_id, buffer);
 		xfree(buffer);
 	}
 
-	job_parse_list_t jobs_rc = _parse_job_list(query, NULL, errors,
-						   true);
+	jobs_rc = _parse_job_list(query, NULL, errors, true);
 
 	if (jobs_rc.rc) {
-		_job_error("%s: job parsing failed for %s",
-			   __func__, context_id);
+		resp_error(errors, jobs_rc.rc, "_parse_job_list",
+			   "job parsing failed for %s", context_id);
 	} else {
 		debug3("%s: job parsing successful for %s",
 		       __func__, context_id);
 		rc = jobs_rc.rc;
 		if (jobs_rc.het_job) {
-			_job_error("%s: unexpected het job request from %s",
-				   __func__, context_id);
+			resp_error(errors, rc, "_parse_job_list",
+				   "unexpected het job request from %s",
+				   context_id);
 		} else {
 			job_array_resp_msg_t *resp = NULL;
 			errno = 0;
-			jobs_rc.job->job_id = job_id;
-			debug5("%s: sending job_id:%d update for %s",
+			jobs_rc.job->job_id_str = xstrdup(job_id);
+			debug5("%s: sending JobId=%s update for %s",
 			       __func__, job_id, context_id);
 
 			rc = slurm_update_job2(jobs_rc.job, &resp);
 
-			if (rc)
-				_job_error("%s: job update from %s failed: %s",
-					   __func__, context_id,
-					   slurm_strerror(errno));
-			else if (resp && resp->error_code && resp->error_code)
-				_job_error(
-					"%s: job array update from %s failed with error_code: %s",
-					__func__, context_id,
-					slurm_strerror(*resp->error_code));
+			if (rc) {
+				resp_error(errors, errno, "_parse_job_list",
+					   "job update from %s failed",
+					   context_id);
+			} else if (resp && resp->error_code &&
+				   resp->error_code) {
+				resp_error(errors, *resp->error_code,
+					   "_parse_job_list",
+					   "job array update from %s failed",
+					   context_id);
+			}
 
 			slurm_free_job_desc_msg(jobs_rc.job);
 			slurm_free_job_array_resp(resp);
@@ -1161,33 +1204,42 @@ static int _op_handler_job(const char *context_id, http_request_method_t method,
 {
 	int rc = SLURM_SUCCESS;
 	data_t *data_jobid;
-	uint32_t job_id = 0;
+
+	const char *job_id_str = NULL;
 	data_t *errors = populate_response_format(resp);
 	debug4("%s: job handler %s called by %s with tag %d",
 	       __func__, get_http_method_string(method), context_id, tag);
 
-	if (!parameters)
-		_job_error("%s: [%s] missing request parameters",
-			   __func__, context_id);
-	else if (!(data_jobid = data_key_get(parameters, "job_id")))
-		_job_error("%s: [%s] missing job_id in parameters",
-			   __func__, context_id);
-	else if (data_get_type(data_jobid) != DATA_TYPE_INT_64)
-		_job_error("%s: [%s] invalid job_id data type",
-			   __func__, context_id);
-	else if (data_get_int(data_jobid) == 0)
-		_job_error("%s: [%s] job_id is zero",
-			   __func__, context_id);
-	else if (data_get_int(data_jobid) >= NO_VAL)
-		_job_error("%s: [%s] job_id too large: %" PRId64,
-			   __func__, context_id, data_get_int(data_jobid));
-	else
-		job_id = data_get_int(data_jobid);
+	if (!parameters) {
+		return resp_error(errors, ESLURM_REST_INVALID_QUERY,
+				  "HTTP request",
+				  "[%s] missing request parameters",
+				  context_id);
+	} else if (!(data_jobid = data_key_get(parameters, "job_id"))) {
+		return resp_error(errors, ESLURM_REST_INVALID_QUERY,
+				  "HTTP request",
+				  "[%s] missing job_id in parameters",
+				  context_id);
+	} else if (data_convert_type(data_jobid, DATA_TYPE_STRING) !=
+		   DATA_TYPE_STRING) {
+		return resp_error(errors, ESLURM_REST_INVALID_QUERY,
+				  "HTTP request",
+				  "[%s] job_id is invalid",
+				  context_id);
+	} else {
+		job_id_str = data_get_string(data_jobid);
+	}
 
 	if (rc) {
+		/* do nothing */
+	} else if (!job_id_str || !job_id_str[0]) {
+		return resp_error(errors, ESLURM_REST_INVALID_QUERY,
+				  "HTTP request",
+				  "[%s] job_id is empty",
+				  context_id);
 	} else if (tag == URL_TAG_JOB && method == HTTP_REQUEST_GET) {
 		rc = _handle_job_get(context_id, method, parameters, query, tag,
-				     resp, job_id, errors);
+				     resp, job_id_str, errors);
 	} else if (tag == URL_TAG_JOB &&
 		   method == HTTP_REQUEST_DELETE) {
 		int signal = 0;
@@ -1200,18 +1252,23 @@ static int _op_handler_job(const char *context_id, http_request_method_t method,
 		else
 			signal = SIGKILL;
 
-		if (signal < 1 || signal >= SIGRTMAX)
-			_job_error("%s: invalid signal: %d", __func__, signal);
-		else
+		if (signal < 1 || signal >= SIGRTMAX) {
+			rc = resp_error(errors, ESLURM_REST_INVALID_QUERY,
+					"HTTP request: signal",
+					"invalid signal: %d", signal);
+		} else {
 			rc = _handle_job_delete(context_id, method, parameters,
-						query, tag, resp, job_id,
+						query, tag, resp, job_id_str,
 						errors, signal);
+		}
 	} else if (tag == URL_TAG_JOB &&
 		   method == HTTP_REQUEST_POST) {
 		rc = _handle_job_post(context_id, method, parameters, query,
-				      tag, resp, job_id, errors);
+				      tag, resp, job_id_str, errors);
 	} else {
-		_job_error("%s: unknown request", __func__);
+		rc = resp_error(errors, ESLURM_REST_INVALID_QUERY,
+				"HTTP request", "%s: unknown request",
+				context_id);
 	}
 
 	return rc;
@@ -1350,8 +1407,8 @@ finish:
 
 static int _op_handler_submit_job(const char *context_id,
 				  http_request_method_t method,
-				  data_t *parameters, data_t *query, int tag,
-				  data_t *resp, void *auth)
+				  data_t *parameters, data_t *query,
+				  int tag, data_t *resp, void *auth)
 {
 	int rc = SLURM_SUCCESS;
 	data_t *errors = populate_response_format(resp);
@@ -1407,12 +1464,12 @@ extern void init_op_jobs(void)
 			      __func__);
 	}
 
-	bind_operation_handler("/slurm/v0.0.36/jobs/", _op_handler_jobs,
-			       URL_TAG_JOBS);
-	bind_operation_handler("/slurm/v0.0.36/job/{job_id}", _op_handler_job,
-			       URL_TAG_JOB);
-	bind_operation_handler("/slurm/v0.0.36/job/submit",
+	bind_operation_handler("/slurm/v0.0.38/job/submit",
 			       _op_handler_submit_job, URL_TAG_JOB_SUBMIT);
+	bind_operation_handler("/slurm/v0.0.38/jobs/", _op_handler_jobs,
+			       URL_TAG_JOBS);
+	bind_operation_handler("/slurm/v0.0.38/job/{job_id}", _op_handler_job,
+			       URL_TAG_JOB);
 }
 
 extern void destroy_op_jobs(void)
