@@ -55,8 +55,9 @@
 #include "src/common/fd.h"
 #include "src/common/hostlist.h"
 #include "src/common/list.h"
-#include "src/common/node_select.h"
 #include "src/common/pack.h"
+#include "src/common/select.h"
+#include "src/common/slurm_protocol_pack.h"
 #include "src/common/slurm_resource_info.h"
 #include "src/common/uid.h"
 #include "src/common/xstring.h"
@@ -66,6 +67,7 @@
 #include "src/slurmctld/groups.h"
 #include "src/slurmctld/licenses.h"
 #include "src/slurmctld/locks.h"
+#include "src/slurmctld/power_save.h"
 #include "src/slurmctld/proc_req.h"
 #include "src/slurmctld/read_config.h"
 #include "src/slurmctld/reservation.h"
@@ -75,8 +77,17 @@
 /* No need to change we always pack SLURM_PROTOCOL_VERSION */
 #define PART_STATE_VERSION        "PROTOCOL_VERSION"
 
+typedef struct {
+	buf_t *buffer;
+	uint32_t parts_packed;
+	bool privileged;
+	uint16_t protocol_version;
+	uint16_t show_flags;
+	uid_t uid;
+	part_record_t **visible_parts;
+} _foreach_pack_part_info_t;
+
 /* Global variables */
-part_record_t default_part;		/* default configuration values */
 List part_list = NULL;			/* partition list */
 char *default_part_name = NULL;		/* name of default partition */
 part_record_t *default_part_loc = NULL;	/* default partition location */
@@ -102,9 +113,8 @@ static int _calc_part_tres(void *x, void *arg)
 	tres_cnt = part_ptr->tres_cnt;
 
 	/* sum up nodes' tres in the partition. */
-	node_ptr = node_record_table_ptr;
-	for (i = 0; i < node_record_count; i++, node_ptr++) {
-		if (!bit_test(part_ptr->node_bitmap, i))
+	for (i = 0; (node_ptr = next_node(&i)); i++) {
+		if (!bit_test(part_ptr->node_bitmap, node_ptr->index))
 			continue;
 		for (j = 0; j < slurmctld_tres_cnt; j++)
 			tres_cnt[j] += node_ptr->tres_cnt[j];
@@ -161,10 +171,11 @@ extern void set_partition_tres()
  */
 extern int build_part_bitmap(part_record_t *part_ptr)
 {
+	int rc = SLURM_SUCCESS;
 	char *this_node_name;
 	bitstr_t *old_bitmap;
 	node_record_t *node_ptr;
-	hostlist_t host_list;
+	hostlist_t host_list, missing_hostlist = NULL;
 	int i;
 
 	part_ptr->total_cpus = 0;
@@ -181,43 +192,51 @@ extern int build_part_bitmap(part_record_t *part_ptr)
 			   node_record_count - 1);
 	}
 
-	if (part_ptr->nodes == NULL) {	/* no nodes in partition */
+	if (!(host_list = nodespec_to_hostlist(part_ptr->orig_nodes,
+					       &part_ptr->nodesets))) {
+		/* Error, restore original bitmap */
+		FREE_NULL_BITMAP(part_ptr->node_bitmap);
+		part_ptr->node_bitmap = old_bitmap;
+		return ESLURM_INVALID_NODE_NAME;
+	} else if (!hostlist_count(host_list)) {
+		info("%s: No nodes in partition %s", __func__, part_ptr->name);
+		/*
+		 * Clear "nodes" but leave "orig_nodes" intact.
+		 * e.g.
+		 * orig_nodes="nodeset1" and all of the nodes in "nodeset1" are
+		 * removed. "nodes" should be cleared to show that there are no
+		 * nodes in the partition right now. "orig_nodes" needs to stay
+		 * intact so that when "nodeset1" nodes come back they are added
+		 * to the partition.
+		 */
+		xfree(part_ptr->nodes);
 		_unlink_free_nodes(old_bitmap, part_ptr);
 		FREE_NULL_BITMAP(old_bitmap);
+		FREE_NULL_HOSTLIST(host_list);
 		return 0;
 	}
 
-	if (!xstrcmp(part_ptr->nodes, "ALL")) {
-		bit_nset(part_ptr->node_bitmap, 0, node_record_count - 1);
-		xfree(part_ptr->nodes);
-		part_ptr->nodes = bitmap2node_name(part_ptr->node_bitmap);
-		bit_nclear(part_ptr->node_bitmap, 0, node_record_count - 1);
-	}
-	if ((host_list = hostlist_create(part_ptr->nodes)) == NULL) {
-		FREE_NULL_BITMAP(old_bitmap);
-		error("hostlist_create error on %s, %m",
-		      part_ptr->nodes);
-		return ESLURM_INVALID_NODE_NAME;
-	}
-
 	while ((this_node_name = hostlist_shift(host_list))) {
 		node_ptr = find_node_record_no_alias(this_node_name);
 		if (node_ptr == NULL) {
-			error("%s: invalid node name %s",
-			      __func__, this_node_name);
+			if (!missing_hostlist)
+				missing_hostlist =
+					hostlist_create(this_node_name);
+			else
+				hostlist_push_host(missing_hostlist,
+						   this_node_name);
+			info("%s: invalid node name %s in partition",
+			     __func__, this_node_name);
 			free(this_node_name);
-			FREE_NULL_BITMAP(old_bitmap);
-			hostlist_destroy(host_list);
-			return ESLURM_INVALID_NODE_NAME;
+			rc = ESLURM_INVALID_NODE_NAME;
+			continue;
 		}
 		part_ptr->total_nodes++;
-		part_ptr->total_cpus += node_ptr->config_ptr->cpus;
+		part_ptr->total_cpus += node_ptr->cpus;
 		part_ptr->max_cpu_cnt = MAX(part_ptr->max_cpu_cnt,
-					    node_ptr->config_ptr->cpus);
-		part_ptr->max_core_cnt =
-				MAX(part_ptr->max_core_cnt,
-				    (node_ptr->config_ptr->cores *
-				     node_ptr->config_ptr->tot_sockets));
+					    node_ptr->cpus);
+		part_ptr->max_core_cnt = MAX(part_ptr->max_core_cnt,
+					     node_ptr->tot_cores);
 
 		for (i = 0; i < node_ptr->part_cnt; i++) {
 			if (node_ptr->part_pptr[i] == part_ptr)
@@ -230,19 +249,39 @@ extern int build_part_bitmap(part_record_t *part_ptr)
 			node_ptr->part_pptr[node_ptr->part_cnt-1] = part_ptr;
 		}
 		if (old_bitmap)
-			bit_clear(old_bitmap,
-				  (int) (node_ptr -
-					 node_record_table_ptr));
-		bit_set(part_ptr->node_bitmap,
-			(int) (node_ptr - node_record_table_ptr));
+			bit_clear(old_bitmap, node_ptr->index);
+
+		bit_set(part_ptr->node_bitmap, node_ptr->index);
 		free(this_node_name);
 	}
 	hostlist_destroy(host_list);
 
+	if ((rc == ESLURM_INVALID_NODE_NAME) && missing_hostlist) {
+		/*
+		 * Remove missing node from partition nodes so we don't keep
+		 * trying to remove them.
+		 */
+		hostlist_t hl;
+		char *missing_nodes;
+
+		hl = hostlist_create(part_ptr->orig_nodes);
+		missing_nodes =
+			hostlist_ranged_string_xmalloc(missing_hostlist);
+		hostlist_delete(hl, missing_nodes);
+		xfree(missing_nodes);
+		xfree(part_ptr->orig_nodes);
+		part_ptr->orig_nodes = hostlist_ranged_string_xmalloc(hl);
+		hostlist_destroy(hl);
+
+	}
+	hostlist_destroy(missing_hostlist);
+	xfree(part_ptr->nodes);
+	part_ptr->nodes = bitmap2node_name(part_ptr->node_bitmap);
+
 	_unlink_free_nodes(old_bitmap, part_ptr);
 	last_node_update = time(NULL);
 	FREE_NULL_BITMAP(old_bitmap);
-	return 0;
+	return rc;
 }
 
 /* unlink nodes removed from a partition */
@@ -254,9 +293,8 @@ static void _unlink_free_nodes(bitstr_t *old_bitmap, part_record_t *part_ptr)
 	if (old_bitmap == NULL)
 		return;
 
-	node_ptr = &node_record_table_ptr[0];
-	for (i = 0; i < node_record_count; i++, node_ptr++) {
-		if (bit_test(old_bitmap, i) == 0)
+	for (i = 0; (node_ptr = next_node(&i)); i++) {
+		if (bit_test(old_bitmap, node_ptr->index) == 0)
 			continue;
 		for (j=0; j<node_ptr->part_cnt; j++) {
 			if (node_ptr->part_pptr[j] != part_ptr)
@@ -275,6 +313,39 @@ static void _unlink_free_nodes(bitstr_t *old_bitmap, part_record_t *part_ptr)
 		last_node_update = time(NULL);
 }
 
+/*
+ * Sync with _init_conf_part().
+ *
+ * _init_conf_part() initializes default values from slurm.conf parameters.
+ * After parsing slurm.conf, _build_single_partitionline_info() copies
+ * slurm_conf_partition_t to part_record_t. Default values between
+ * slurm_conf_partition_t and part_record_t should stay in sync in case a
+ * part_record_t is created outside of slurm.conf parsing.
+ */
+static void _init_part_record(part_record_t *part_ptr)
+{
+	part_ptr->magic = PART_MAGIC;
+	if (slurm_conf.conf_flags & CTL_CONF_DRJ)
+		part_ptr->flags |= PART_FLAG_NO_ROOT;
+	part_ptr->max_nodes_orig = INFINITE;
+	part_ptr->min_nodes = 1;
+	part_ptr->min_nodes_orig = 1;
+
+	/* sync with slurm_conf_partition_t */
+	part_ptr->default_time = NO_VAL;
+	part_ptr->max_cpus_per_node = INFINITE;
+	part_ptr->max_nodes = INFINITE;
+	part_ptr->max_share = 1;
+	part_ptr->max_time = INFINITE;
+	part_ptr->over_time_limit = NO_VAL16;
+	part_ptr->preempt_mode = NO_VAL16;
+	part_ptr->priority_job_factor = 1;
+	part_ptr->priority_tier = 1;
+	part_ptr->resume_timeout = NO_VAL16;
+	part_ptr->state_up = PARTITION_UP;
+	part_ptr->suspend_time = NO_VAL;
+	part_ptr->suspend_timeout = NO_VAL16;
+}
 
 /*
  * create_part_record - create a partition record
@@ -287,94 +358,8 @@ part_record_t *create_part_record(const char *name)
 
 	last_part_update = time(NULL);
 
-	part_ptr->magic = PART_MAGIC;
-	part_ptr->name              = xstrdup(name);
-	part_ptr->alternate         = xstrdup(default_part.alternate);
-	part_ptr->cr_type	    = default_part.cr_type;
-	part_ptr->job_defaults_list =
-			job_defaults_copy(default_part.job_defaults_list);
-	part_ptr->flags             = default_part.flags;
-	part_ptr->grace_time 	    = default_part.grace_time;
-	part_ptr->max_share         = default_part.max_share;
-	part_ptr->max_time          = default_part.max_time;
-	part_ptr->default_time      = default_part.default_time;
-	part_ptr->max_cpus_per_node = default_part.max_cpus_per_node;
-	part_ptr->max_nodes         = default_part.max_nodes;
-	part_ptr->max_nodes_orig    = default_part.max_nodes;
-	part_ptr->min_nodes         = default_part.min_nodes;
-	part_ptr->min_nodes_orig    = default_part.min_nodes;
-	part_ptr->over_time_limit   = default_part.over_time_limit;
-	part_ptr->preempt_mode      = default_part.preempt_mode;
-	part_ptr->priority_job_factor = default_part.priority_job_factor;
-	part_ptr->priority_tier     = default_part.priority_tier;
-	part_ptr->state_up          = default_part.state_up;
-
-	if (part_max_priority) {
-		part_ptr->norm_priority =
-			(double)default_part.priority_job_factor /
-			(double)part_max_priority;
-	}
-	part_ptr->node_bitmap       = NULL;
-
-	if (default_part.allow_accounts) {
-		part_ptr->allow_accounts = xstrdup(default_part.allow_accounts);
-		accounts_list_build(part_ptr->allow_accounts,
-				    &part_ptr->allow_account_array);
-	} else
-		part_ptr->allow_accounts = NULL;
-
-	if (default_part.allow_groups)
-		part_ptr->allow_groups = xstrdup(default_part.allow_groups);
-	else
-		part_ptr->allow_groups = NULL;
-
-	if (default_part.allow_qos) {
-		part_ptr->allow_qos = xstrdup(default_part.allow_qos);
-		qos_list_build(part_ptr->allow_qos,
-			       &part_ptr->allow_qos_bitstr);
-	} else
-		part_ptr->allow_qos = NULL;
-
-	if (default_part.deny_accounts) {
-		part_ptr->deny_accounts = xstrdup(default_part.deny_accounts);
-		accounts_list_build(part_ptr->deny_accounts,
-				    &part_ptr->deny_account_array);
-	} else
-		part_ptr->deny_accounts = NULL;
-
-	if (default_part.deny_qos) {
-		part_ptr->deny_qos = xstrdup(default_part.deny_qos);
-		qos_list_build(part_ptr->deny_qos, &part_ptr->deny_qos_bitstr);
-	} else
-		part_ptr->deny_qos = NULL;
-
-	if (default_part.qos_char) {
-		slurmdb_qos_rec_t qos_rec;
-		xfree(part_ptr->qos_char);
-		part_ptr->qos_char = xstrdup(default_part.qos_char);
-
-		memset(&qos_rec, 0, sizeof(slurmdb_qos_rec_t));
-		qos_rec.name = part_ptr->qos_char;
-		if (assoc_mgr_fill_in_qos(
-			    acct_db_conn, &qos_rec, accounting_enforce,
-			    (slurmdb_qos_rec_t **)&part_ptr->qos_ptr, 0)
-		    != SLURM_SUCCESS) {
-			fatal("Partition %s has an invalid qos (%s), "
-			      "please check your configuration",
-			      part_ptr->name, qos_rec.name);
-		}
-	}
-
-	if (default_part.allow_alloc_nodes) {
-		part_ptr->allow_alloc_nodes =
-			xstrdup(default_part.allow_alloc_nodes);
-	}
-
-	if (default_part.nodes)
-		part_ptr->nodes = xstrdup(default_part.nodes);
-	else
-		part_ptr->nodes = NULL;
-	part_ptr->bf_data = NULL;
+	_init_part_record(part_ptr);
+	part_ptr->name = xstrdup(name);
 
 	(void) list_append(part_list, part_ptr);
 
@@ -402,7 +387,7 @@ int dump_all_part_state(void)
 
 	/* write partition records to buffer */
 	lock_slurmctld(part_read_lock);
-	list_for_each(part_list, _dump_part_state, buffer);
+	list_for_each_ro(part_list, _dump_part_state, buffer);
 
 	old_file = xstrdup(slurm_conf.state_save_location);
 	xstrcat(old_file, "/part_state.old");
@@ -511,7 +496,8 @@ static int _dump_part_state(void *x, void *arg)
 	packstr(part_ptr->alternate,     buffer);
 	packstr(part_ptr->deny_accounts, buffer);
 	packstr(part_ptr->deny_qos,      buffer);
-	packstr(part_ptr->nodes,         buffer);
+	/* Save orig_nodes as nodes will be built from orig_nodes */
+	packstr(part_ptr->orig_nodes, buffer);
 
 	return 0;
 }
@@ -751,8 +737,13 @@ int load_all_part_state(void)
 		xfree(part_ptr->deny_qos);
 		part_ptr->deny_qos       = deny_qos;
 		qos_list_build(part_ptr->deny_qos, &part_ptr->deny_qos_bitstr);
+
+		/*
+		 * Store saved nodelist in orig_nodes. nodes will be regenerated
+		 * from orig_nodes.
+		 */
 		xfree(part_ptr->nodes);
-		part_ptr->nodes = nodes;
+		part_ptr->orig_nodes = nodes;
 
 		xfree(part_name);
 	}
@@ -852,58 +843,14 @@ extern List get_part_list(char *name, char **err_part)
 }
 
 /*
- * init_part_conf - initialize the default partition configuration values
- *	and create a (global) partition list.
- * this should be called before creating any partition entries.
- * global: default_part - default partition values
- *         part_list - global partition list
+ * Create a global partition list.
+ *
+ * This should be called before creating any partition entries.
  */
 void init_part_conf(void)
 {
 	last_part_update = time(NULL);
 
-	xfree(default_part.name);	/* needed for reconfig */
-	default_part.name           = xstrdup("DEFAULT");
-	default_part.flags          = 0;
-	if (slurm_conf.conf_flags & CTL_CONF_DRJ)
-		default_part.flags |= PART_FLAG_NO_ROOT;
-	default_part.max_time       = INFINITE;
-	default_part.default_time   = NO_VAL;
-	FREE_NULL_LIST(default_part.job_defaults_list);
-	default_part.max_cpus_per_node = INFINITE;
-	default_part.max_nodes      = INFINITE;
-	default_part.max_nodes_orig = INFINITE;
-	default_part.min_nodes      = 1;
-	default_part.min_nodes_orig = 1;
-	default_part.state_up       = PARTITION_UP;
-	default_part.max_share      = 1;
-	default_part.over_time_limit = NO_VAL16;
-	default_part.preempt_mode   = NO_VAL16;
-	default_part.priority_tier  = 1;
-	default_part.priority_job_factor = 1;
-	default_part.norm_priority  = 0;
-	default_part.total_nodes    = 0;
-	default_part.total_cpus     = 0;
-	default_part.grace_time     = 0;
-	default_part.cr_type	    = 0;
-	xfree(default_part.nodes);
-	xfree(default_part.allow_accounts);
-	accounts_list_free(&default_part.allow_account_array);
-	xfree(default_part.allow_groups);
-	xfree(default_part.allow_qos);
-	xfree(default_part.qos_char);
-	default_part.qos_ptr = NULL;
-	FREE_NULL_BITMAP(default_part.allow_qos_bitstr);
-	xfree(default_part.allow_uids);
-	xfree(default_part.allow_alloc_nodes);
-	xfree(default_part.alternate);
-	xfree(default_part.deny_accounts);
-	accounts_list_free(&default_part.deny_account_array);
-	xfree(default_part.deny_qos);
-	FREE_NULL_BITMAP(default_part.deny_qos_bitstr);
-	FREE_NULL_LIST(default_part.job_defaults_list);
-	FREE_NULL_BITMAP(default_part.node_bitmap);
-
 	if (part_list)		/* delete defunct partitions */
 		list_flush(part_list);
 	else
@@ -946,8 +893,7 @@ static void _list_delete_part(void *part_entry)
 	int i, j, k;
 
 	part_ptr = (part_record_t *) part_entry;
-	node_ptr = &node_record_table_ptr[0];
-	for (i = 0; i < node_record_count; i++, node_ptr++) {
+	for (i = 0; (node_ptr = next_node(&i)); i++) {
 		for (j=0; j<node_ptr->part_cnt; j++) {
 			if (node_ptr->part_pptr[j] != part_ptr)
 				continue;
@@ -976,7 +922,9 @@ static void _list_delete_part(void *part_entry)
 	FREE_NULL_BITMAP(part_ptr->deny_qos_bitstr);
 	FREE_NULL_LIST(part_ptr->job_defaults_list);
 	xfree(part_ptr->name);
+	xfree(part_ptr->orig_nodes);
 	xfree(part_ptr->nodes);
+	xfree(part_ptr->nodesets);
 	FREE_NULL_BITMAP(part_ptr->node_bitmap);
 	xfree(part_ptr->qos_char);
 	xfree(part_ptr->tres_cnt);
@@ -1017,12 +965,11 @@ static int _match_part_ptr(void *part_ptr, void *key)
 }
 
 /* partition is visible to the user */
-extern bool part_is_visible(part_record_t *part_ptr, uid_t uid)
+static bool _part_is_visible(part_record_t *part_ptr, uid_t uid)
 {
 	xassert(verify_lock(PART_LOCK, READ_LOCK));
+	xassert(uid != 0);
 
-	if (validate_operator(uid))
-		return true;
 	if (part_ptr->flags & PART_FLAG_HIDDEN)
 		return false;
 	if (validate_group(part_ptr, uid) == 0)
@@ -1031,20 +978,93 @@ extern bool part_is_visible(part_record_t *part_ptr, uid_t uid)
 	return true;
 }
 
-/* partition is visible to the user */
-extern bool part_is_visible_user_rec(part_record_t *part_ptr,
-				     slurmdb_user_rec_t *user)
+typedef struct {
+	uid_t uid;
+	part_record_t **visible_parts;
+} build_visible_parts_arg_t;
+
+static int _build_visible_parts_foreach(void *elem, void *x)
 {
-	xassert(verify_lock(PART_LOCK, READ_LOCK));
+	part_record_t *part_ptr = elem;
+	build_visible_parts_arg_t *arg = x;
 
-	if (part_ptr->flags & PART_FLAG_HIDDEN)
-		return false;
-	if (!validate_group(part_ptr, user->uid))
-		return false;
+	if (_part_is_visible(part_ptr, arg->uid)) {
+		*(arg->visible_parts) = part_ptr;
+		arg->visible_parts++;
+		if (get_log_level() >= LOG_LEVEL_DEBUG3) {
+			char *tmp_str = NULL;
+			for (int i = 0; arg->visible_parts[i]; i++)
+				xstrfmtcat(tmp_str, "%s%s", tmp_str ? "," : "",
+					   arg->visible_parts[i]->name);
+			debug3("%s: uid:%u visible_parts:%s",
+			       __func__, arg->uid, tmp_str);
+			xfree(tmp_str);
+		}
+	}
 
+	return SLURM_SUCCESS;
+}
+
+extern part_record_t **build_visible_parts(uid_t uid, bool skip)
+{
+	part_record_t **visible_parts_save;
+	part_record_t **visible_parts;
+	build_visible_parts_arg_t args = {0};
+
+	/*
+	 * The array of visible parts isn't used for privileged (i.e. operators)
+	 * users or when SHOW_ALL is requested, so no need to create list.
+	 */
+	if (skip)
+		return NULL;
+
+	visible_parts = xcalloc(list_count(part_list) + 1,
+				sizeof(part_record_t *));
+	args.uid = uid;
+	args.visible_parts = visible_parts;
+
+	/*
+	 * Save start pointer to start of the list so can point to start
+	 * after appending to the list.
+	 */
+	visible_parts_save = visible_parts;
+	list_for_each(part_list, _build_visible_parts_foreach, &args);
+
+	return visible_parts_save;
+}
+
+extern int part_not_on_list(part_record_t **parts, part_record_t *x)
+{
+	for (int i = 0; parts[i]; i++) {
+		if (parts[i] == x) {
+			debug3("%s: partition: %s on visible part list",
+			       __func__, x->name);
+			return false;
+		} else
+			debug3("%s: partition: %s not on visible part list",
+			       __func__, x->name);
+	}
 	return true;
 }
 
+static int _pack_part(void *object, void *arg)
+{
+	part_record_t *part_ptr = object;
+	_foreach_pack_part_info_t *pack_info = arg;
+
+	xassert(part_ptr->magic == PART_MAGIC);
+
+	if (!(pack_info->show_flags & SHOW_ALL) &&
+	    !pack_info->privileged &&
+	    part_not_on_list(pack_info->visible_parts, part_ptr))
+		return SLURM_SUCCESS;
+
+	pack_part(part_ptr, pack_info->buffer, pack_info->protocol_version);
+	pack_info->parts_packed++;
+
+	return SLURM_SUCCESS;
+}
+
 /*
  * pack_all_part - dump all partition information for all partitions in
  *	machine independent form (for network transmission)
@@ -1060,43 +1080,37 @@ extern void pack_all_part(char **buffer_ptr, int *buffer_size,
 			  uint16_t show_flags, uid_t uid,
 			  uint16_t protocol_version)
 {
-	ListIterator part_iterator;
-	part_record_t *part_ptr;
-	uint32_t parts_packed;
 	int tmp_offset;
-	buf_t *buffer;
 	time_t now = time(NULL);
+	bool privileged = validate_operator(uid);
+	_foreach_pack_part_info_t pack_info = {
+		.buffer = init_buf(BUF_SIZE),
+		.parts_packed = 0,
+		.privileged = privileged,
+		.protocol_version = protocol_version,
+		.show_flags = show_flags,
+		.uid = uid,
+		.visible_parts = build_visible_parts(uid, privileged),
+	};
 
 	buffer_ptr[0] = NULL;
 	*buffer_size = 0;
 
-	buffer = init_buf(BUF_SIZE);
-
 	/* write header: version and time */
-	parts_packed = 0;
-	pack32(parts_packed, buffer);
-	pack_time(now, buffer);
+	pack32(0, pack_info.buffer);
+	pack_time(now, pack_info.buffer);
 
-	/* write individual partition records */
-	part_iterator = list_iterator_create(part_list);
-	while ((part_ptr = list_next(part_iterator))) {
-		xassert (part_ptr->magic == PART_MAGIC);
-		if (((show_flags & SHOW_ALL) == 0) &&
-		    !part_is_visible(part_ptr, uid))
-			continue;
-		pack_part(part_ptr, buffer, protocol_version);
-		parts_packed++;
-	}
-	list_iterator_destroy(part_iterator);
+	list_for_each_ro(part_list, _pack_part, &pack_info);
 
 	/* put the real record count in the message body header */
-	tmp_offset = get_buf_offset(buffer);
-	set_buf_offset(buffer, 0);
-	pack32(parts_packed, buffer);
-	set_buf_offset(buffer, tmp_offset);
+	tmp_offset = get_buf_offset(pack_info.buffer);
+	set_buf_offset(pack_info.buffer, 0);
+	pack32(pack_info.parts_packed, pack_info.buffer);
+	set_buf_offset(pack_info.buffer, tmp_offset);
 
-	*buffer_size = get_buf_offset(buffer);
-	buffer_ptr[0] = xfer_buf_data(buffer);
+	*buffer_size = get_buf_offset(pack_info.buffer);
+	buffer_ptr[0] = xfer_buf_data(pack_info.buffer);
+	xfree(pack_info.visible_parts);
 }
 
 
@@ -1112,7 +1126,54 @@ extern void pack_all_part(char **buffer_ptr, int *buffer_size,
  */
 void pack_part(part_record_t *part_ptr, buf_t *buffer, uint16_t protocol_version)
 {
-	if (protocol_version >= SLURM_21_08_PROTOCOL_VERSION) {
+	if (protocol_version >= SLURM_22_05_PROTOCOL_VERSION) {
+		if (default_part_loc == part_ptr)
+			part_ptr->flags |= PART_FLAG_DEFAULT;
+		else
+			part_ptr->flags &= (~PART_FLAG_DEFAULT);
+
+		packstr(part_ptr->name, buffer);
+		pack32(part_ptr->cpu_bind, buffer);
+		pack32(part_ptr->grace_time, buffer);
+		pack32(part_ptr->max_time, buffer);
+		pack32(part_ptr->default_time, buffer);
+		pack32(part_ptr->max_nodes_orig, buffer);
+		pack32(part_ptr->min_nodes_orig, buffer);
+		pack32(part_ptr->total_nodes, buffer);
+		pack32(part_ptr->total_cpus, buffer);
+		pack64(part_ptr->def_mem_per_cpu, buffer);
+		pack32(part_ptr->max_cpus_per_node, buffer);
+		pack64(part_ptr->max_mem_per_cpu, buffer);
+
+		pack16(part_ptr->flags,      buffer);
+		pack16(part_ptr->max_share,  buffer);
+		pack16(part_ptr->over_time_limit, buffer);
+		pack16(part_ptr->preempt_mode, buffer);
+		pack16(part_ptr->priority_job_factor, buffer);
+		pack16(part_ptr->priority_tier, buffer);
+		pack16(part_ptr->state_up, buffer);
+		pack16(part_ptr->cr_type, buffer);
+		pack16(part_ptr->resume_timeout, buffer);
+		pack16(part_ptr->suspend_timeout, buffer);
+		pack32(part_ptr->suspend_time, buffer);
+
+		packstr(part_ptr->allow_accounts, buffer);
+		packstr(part_ptr->allow_groups, buffer);
+		packstr(part_ptr->allow_alloc_nodes, buffer);
+		packstr(part_ptr->allow_qos, buffer);
+		packstr(part_ptr->qos_char, buffer);
+		packstr(part_ptr->alternate, buffer);
+		packstr(part_ptr->deny_accounts, buffer);
+		packstr(part_ptr->deny_qos, buffer);
+		packstr(part_ptr->nodes, buffer);
+		packstr(part_ptr->nodesets, buffer);
+		pack_bit_str_hex(part_ptr->node_bitmap, buffer);
+		packstr(part_ptr->billing_weights_str, buffer);
+		packstr(part_ptr->tres_fmt_str, buffer);
+		(void)slurm_pack_list(part_ptr->job_defaults_list,
+				      job_defaults_pack, buffer,
+				      protocol_version);
+	} else if (protocol_version >= SLURM_21_08_PROTOCOL_VERSION) {
 		if (default_part_loc == part_ptr)
 			part_ptr->flags |= PART_FLAG_DEFAULT;
 		else
@@ -1722,6 +1783,8 @@ extern int update_part(update_part_msg_t * part_desc, bool create_flag)
 					part_ptr->nodes[i] = ',';
 			}
 		}
+		xfree(part_ptr->orig_nodes);
+		part_ptr->orig_nodes = xstrdup(part_ptr->nodes);
 
 		error_code = build_part_bitmap(part_ptr);
 		if (error_code) {
@@ -1733,6 +1796,7 @@ extern int update_part(update_part_msg_t * part_desc, bool create_flag)
 			xfree(backup_node_list);
 		}
 		update_part_nodes_in_resv(part_ptr);
+		power_save_set_timeouts(NULL);
 
 		assoc_mgr_lock(&assoc_tres_read_lock);
 		_calc_part_tres(part_ptr, NULL);
@@ -2001,8 +2065,6 @@ void load_part_uid_allow_list(int force)
 void part_fini (void)
 {
 	FREE_NULL_LIST(part_list);
-	xfree(default_part_name);
-	xfree(default_part.name);
 	default_part_loc = NULL;
 }
 
